SystemOrganization addCategory: #'Seaside-Canvas'!
SystemOrganization addCategory: #'Seaside-Canvas-Tags'!

WAObject subclass: #WABrush
	instanceVariableNames: 'canvas parent closed'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Seaside-Canvas'!

!WABrush commentStamp: 'lr 3/27/2007 11:18' prior: 0!
I represent a brush to be used on a *WACanvas*.

Instance Variables
	canvas:		The canvas instance I am used with.
	parent:		The parent brush I am used within.
	closed:		Wether I have been closed/flushed yet.!

!WABrush methodsFor: 'public' stamp: 'lr 4/9/2007 16:32'!
close
	"Close and flush the receiver onto the canvas and its associated document."

	closed ifFalse: [ self with: nil ]! !

!WABrush methodsFor: 'initialize-release' stamp: 'jf 8/30/2008 16:29'!
initialize
	super initialize.
	closed := false! !

!WABrush methodsFor: 'private' stamp: 'lr 3/27/2007 17:38'!
labelForSelector: aSymbol of: anObject
	^ anObject labelForSelector: aSymbol! !

!WABrush methodsFor: 'accessing' stamp: 'avi 6/13/2004 00:27'!
parent
	^ parent! !

!WABrush methodsFor: 'initialize-release' stamp: 'avi 6/13/2004 00:27'!
setParent: aBrush canvas: aCanvas
	parent := aBrush.
	canvas := aCanvas! !

!WABrush methodsFor: 'public' stamp: 'lr 3/27/2007 11:10'!
with: aBlock
	canvas nest: aBlock.
	closed := true! !

WABrush subclass: #WACompound
	instanceVariableNames: 'callbackBlock value id properties'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Seaside-Canvas-Tags'!

!WACompound commentStamp: 'pmm 9/22/2007 22:22' prior: 0!
I am the superclass for stuff that is not html element but you still want to access via the canvas API.!

!WACompound methodsFor: 'callbacks' stamp: 'lr 10/28/2007 14:42'!
callback: aBlock
	callbackBlock := aBlock! !

!WACompound methodsFor: 'public' stamp: 'lr 3/27/2007 18:00'!
close
	closed ifTrue: [ ^ self ].
	closed := true.
	self with: nil! !

!WACompound methodsFor: 'testing' stamp: 'lr 3/27/2007 17:25'!
hasCallback
	^ callbackBlock notNil! !

!WACompound methodsFor: 'accessing' stamp: 'lr 3/27/2007 17:55'!
id
	^ id! !

!WACompound methodsFor: 'accessing' stamp: 'lr 3/27/2007 17:55'!
id: aString
	id := aString! !

!WACompound methodsFor: 'initialize-release' stamp: 'lr 3/27/2007 17:51'!
initialize
	super initialize.
	"do not remove properties, it is used by SeasideAsync"
	properties := Dictionary new! !

!WACompound methodsFor: 'callbacks' stamp: 'lr 3/29/2008 16:40'!
on: aSelector of: anObject
	self value: (anObject perform: aSelector).
	self callback: (MessageSend receiver: anObject selector: aSelector asMutator)! !

!WACompound methodsFor: 'accessing' stamp: 'lr 3/27/2007 17:25'!
value
	^ value! !

!WACompound methodsFor: 'accessing' stamp: 'mb 1/30/2006 18:40'!
value: aValue
	value := aValue! !

WACompound subclass: #WADateInput
	instanceVariableNames: 'options month day year'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Seaside-Canvas-Tags'!

!WADateInput commentStamp: 'pmm 4/30/2007 23:41' prior: 0!
I am a composition of three input fields that allow the user to select year, month and day. #options: can be used to toggle them individually.
The argument for my callback blocks is an instance of Date.

See WADateTimeTest >> #renderDateTimeOn:!

!WADateInput methodsFor: 'private rendering' stamp: 'pmm 11/4/2008 20:27'!
addCallback
	self hasCallback ifFalse: [
		canvas flush.
		^ self ].
	canvas hiddenInput
		callback: [ callbackBlock value: self setValueWithNewDate ];
		 with: nil! !

!WADateInput methodsFor: 'private callbacks' stamp: 'lr 10/28/2007 14:42'!
addDayCallbackToBrush: aBrush
	self hasCallback ifFalse: [^self].
	aBrush on: #day of: self! !

!WADateInput methodsFor: 'private callbacks' stamp: 'lr 10/28/2007 14:42'!
addMonthCallbackToBrush: aBrush
	self hasCallback ifFalse: [^self].
	aBrush on: #month of: self! !

!WADateInput methodsFor: 'private callbacks' stamp: 'mb 11/3/2006 14:19'!
addYearCallbackToBrush: aBrush
	self hasCallback ifFalse: [^self].
	aBrush on: #year of: self! !

!WADateInput methodsFor: 'private callbacks' stamp: 'mb 11/4/2006 10:20'!
day
	^day! !

!WADateInput methodsFor: 'private callbacks' stamp: 'pmm 12/9/2007 12:31'!
day: anIntegerOrString
	day := [ anIntegerOrString asInteger ] on: Error do: [ :e | 1 ]! !

!WADateInput methodsFor: 'private' stamp: 'mb 11/3/2006 09:57'!
defaultOptions
	^#(month day year)! !

!WADateInput methodsFor: 'private callbacks' stamp: 'mb 11/4/2006 10:20'!
month
	^month! !

!WADateInput methodsFor: 'private callbacks' stamp: 'pmm 12/9/2007 12:31'!
month: anIntegerOrString
	month := [ anIntegerOrString asInteger ] on: Error do: [ :error | 1 ]! !

!WADateInput methodsFor: 'accessing' stamp: 'lr 10/28/2007 14:42'!
options
	^options ifNil: [ self defaultOptions ]! !

!WADateInput methodsFor: 'accessing' stamp: 'mb 11/3/2006 09:58'!
options: anArray
	"Valid values in the array : #year #month #day
	Can be used to control what is shown and in what order
	Default #(month day year)"
	options := anArray! !

!WADateInput methodsFor: 'private rendering' stamp: 'mb 7/10/2007 09:42'!
renderDay
	| brush |
	brush := canvas textInput
				id: (self id isNil
						ifFalse: [self id , '-day']);
				 value: day;
				 yourself.
	self addDayCallbackToBrush: brush.
	brush attributeAt: 'size' put: 2;
		 attributeAt: 'maxlength' put: 2! !

!WADateInput methodsFor: 'private rendering' stamp: 'lr 10/28/2007 14:42'!
renderMonth
	| brush |
	brush := canvas select
		id: self id;
		list: (1 to: 12);
		selected: month;
		yourself.
	self addMonthCallbackToBrush: brush.
	brush
		labels: [:ea | Date nameOfMonth: ea]! !

!WADateInput methodsFor: 'private rendering' stamp: 'lr 10/28/2007 14:42'!
renderYear
	| brush |
	brush := (canvas textInput)
		id: (self id isNil ifTrue: [nil] ifFalse: [self id , '-year']);
		value: year;
		yourself.
	self addYearCallbackToBrush: brush.
	brush
		attributeAt: 'size' put: 4;
		attributeAt: 'maxlength' put: 4! !

!WADateInput methodsFor: 'private callbacks' stamp: 'dkh 3/10/2008 10:50'!
setValueWithNewDate
	^value := Date
		newDay: ((day min: (Date daysInMonthNumber: month forYear: year)) max: 1)
		monthNumber: month
		year: year! !

!WADateInput methodsFor: 'public' stamp: 'lr 10/28/2007 14:42'!
with: aBlock
	value isNil ifTrue: [value := Date today].
	day := value dayOfMonth.
	month := value monthIndex.
	year := value year.
	self options
		do: [:each | self perform: ('render' , each asLowercase capitalized) asSymbol ]
		separatedBy: [ canvas space ].
	self addCallback! !

!WADateInput methodsFor: 'private callbacks' stamp: 'mb 11/4/2006 10:20'!
year
	^year! !

!WADateInput methodsFor: 'private callbacks' stamp: 'pmm 12/9/2007 12:30'!
year: anIntegerOrString
	year := [ anIntegerOrString asInteger ] on: Error do: [ :e | 1900 ]! !

WACompound subclass: #WATimeInput
	instanceVariableNames: 'withSeconds hours minutes seconds'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Seaside-Canvas-Tags'!

!WATimeInput commentStamp: 'pmm 4/30/2007 23:41' prior: 0!
I am a composition of three input fields that allows the user to select hour, minute and optionally second.
#withSeconds and #withoutSeconds can be used to toggle seconds.

The argument for my callback blocks is an instance of Time.

See WADateTimeTest >> #renderDateTimeOn:!

!WATimeInput methodsFor: 'private rendering' stamp: 'pmm 11/4/2008 20:27'!
addCallback
	self hasCallback ifFalse: [
		canvas flush.
		^ self ].
	canvas hiddenInput
		callback: [ callbackBlock value: self setValueWithNewTime ];
		 with: nil! !

!WATimeInput methodsFor: 'private callbacks' stamp: 'lr 10/28/2007 14:42'!
addHoursCallbackToBrush: aBrush
	self hasCallback ifFalse: [^self].
	aBrush on: #hours of: self! !

!WATimeInput methodsFor: 'private callbacks' stamp: 'lr 10/28/2007 14:42'!
addMinutesCallbackToBrush: aBrush
	self hasCallback ifFalse: [^self].
	aBrush on: #minutes of: self! !

!WATimeInput methodsFor: 'private callbacks' stamp: 'lr 10/28/2007 14:42'!
addSecondsCallbackToBrush: aBrush
	self hasCallback ifFalse: [^self].
	aBrush on: #seconds of: self! !

!WATimeInput methodsFor: 'private callbacks' stamp: 'lr 10/28/2007 14:42'!
hours
	^hours! !

!WATimeInput methodsFor: 'private callbacks' stamp: 'pmm 12/9/2007 12:29'!
hours: anIntegerOrString
	hours := [ anIntegerOrString asInteger ] on: Error do: [ :e | 0 ]! !

!WATimeInput methodsFor: 'initialize-release' stamp: 'pmm 9/12/2006 19:18'!
initialize
	super initialize.
	self withoutSeconds! !

!WATimeInput methodsFor: 'private callbacks' stamp: 'mb 11/4/2006 10:21'!
minutes
	^minutes! !

!WATimeInput methodsFor: 'private callbacks' stamp: 'pmm 12/9/2007 12:29'!
minutes: anIntegerOrString
	minutes := [ anIntegerOrString asInteger ] on: Error do: [ :error | 0]! !

!WATimeInput methodsFor: 'private rendering' stamp: 'lr 3/27/2007 17:28'!
renderHours
	| brush |
	brush := canvas textInput
		id: self id;
		value: hours asTwoCharacterString;
		yourself.
	self addHoursCallbackToBrush: brush.
	brush
		attributeAt: 'size' put: 2;
		attributeAt: 'maxlength' put: 2! !

!WATimeInput methodsFor: 'private rendering' stamp: 'mb 7/10/2007 09:42'!
renderMinutes
	| brush |
	brush := canvas textInput
				id: (self id isNil
						ifFalse: [self id , '-mins']);
				 value: minutes asTwoCharacterString;
				 yourself.
	self addMinutesCallbackToBrush: brush.
	brush attributeAt: 'size' put: 2;
		 attributeAt: 'maxlength' put: 2! !

!WATimeInput methodsFor: 'private rendering' stamp: 'mb 7/10/2007 09:42'!
renderSeconds
	| brush |
	brush := canvas textInput
				id: (self id isNil
						ifFalse: [self id , '-secs']);
				 value: seconds asTwoCharacterString;
				 yourself.
	self addSecondsCallbackToBrush: brush.
	brush attributeAt: 'size' put: 2;
		 attributeAt: 'maxlength' put: 2! !

!WATimeInput methodsFor: 'private callbacks' stamp: 'mb 11/4/2006 10:21'!
seconds
	^seconds! !

!WATimeInput methodsFor: 'private callbacks' stamp: 'lr 10/28/2007 14:42'!
seconds: anIntegerOrString
	seconds := [anIntegerOrString asNumber] on: Error do: [:e | 0]! !

!WATimeInput methodsFor: 'private callbacks' stamp: 'lr 10/28/2007 14:42'!
setValueWithNewTime
	^value := Time
				hour: hours
				minute: (minutes min: 59)
				second: (seconds min: 59)! !

!WATimeInput methodsFor: 'public' stamp: 'lr 10/28/2007 14:42'!
with: aBlock
	value isNil ifTrue: [value := Time now].
	hours := value hour.
	minutes := value minutes.
	seconds := value seconds.
	self renderHours.
	canvas space; text: ':'; space.
	self renderMinutes.
	withSeconds ifTrue:
		[canvas space; text: ':'; space.
		self renderSeconds]
	ifFalse:
		[seconds := 0].
	self addCallback! !

!WATimeInput methodsFor: 'accessing' stamp: 'mb 2/6/2006 09:55'!
withSeconds
	withSeconds := true! !

!WATimeInput methodsFor: 'accessing' stamp: 'mb 2/6/2006 09:56'!
withoutSeconds
	withSeconds := false! !

WABrush subclass: #WATagBrush
	instanceVariableNames: 'attributes'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Seaside-Canvas'!

!WATagBrush commentStamp: 'pmm 4/15/2007 22:14' prior: 0!
This is the superclass for all XML element classes. Its main addtions are
- element name (#tag)
- attributes (instance of WAHtmlAttributes)
- common events (onXXX), this is a hack and would better be solved with traits!

WATagBrush subclass: #WAAnchorTag
	instanceVariableNames: 'url'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Seaside-Canvas-Tags'!

!WAAnchorTag commentStamp: 'lr 7/4/2007 20:42' prior: 0!
I'm the class responsible for adding anchors (links) to your webpage. There are multiple ways of using me.

1. In the following case, the method #doSomethingOnClick will be sent to self when the user click on the anchor 'Click here to do something':

	html anchor
		callback: [ self doSomethingOnClick ];
		with: 'Click here to do something'.
		
The following code is a shortcut to create an anchor. The executed action is #doSomethingOnClick and the text is 'Do Something On Click':

	html anchor
		on: #doSomethingOnClick of: self

2. In the following case, a link to an external resource will be generated:

	html anchor
		url: 'http://www.seaside.st';
		with: 'Visit the Seaside'!

!WAAnchorTag methodsFor: 'callbacks' stamp: 'lr 8/17/2007 15:07'!
callback: aBlock
	self url addParameter: (canvas callbacks registerActionCallback: aBlock)! !

!WAAnchorTag methodsFor: 'deprecated' stamp: 'lr 8/17/2007 15:42'!
canHaveShortcut
	^ true! !

!WAAnchorTag methodsFor: 'documents' stamp: 'lr 8/17/2007 15:08'!
document: aDocument
	self document: aDocument mimeType: nil! !

!WAAnchorTag methodsFor: 'documents' stamp: 'lr 8/17/2007 15:08'!
document: aDocument mimeType: mimeType
	self document: aDocument mimeType: mimeType fileName: nil! !

!WAAnchorTag methodsFor: 'documents' stamp: 'lr 8/17/2007 15:09'!
document: aDocument mimeType: mimeType fileName: fileName
	self url: (canvas context urlForDocument: aDocument mimeType: mimeType fileName: fileName)! !

!WAAnchorTag methodsFor: 'url' stamp: 'lr 8/17/2007 15:16'!
extraParameters: aCollection
	self url addParameter: aCollection! !

!WAAnchorTag methodsFor: 'url' stamp: 'pmm 10/7/2007 14:32'!
extraPath: aString
	self url addToPath: aString! !

!WAAnchorTag methodsFor: 'url' stamp: 'lr 8/17/2007 15:16'!
fragment: aString
	self url fragment: aString! !

!WAAnchorTag methodsFor: 'deprecated' stamp: 'lr 8/17/2007 15:43'!
ignoreURL
	"does nothing when clicked"
	^ self onClick: 'return false'! !

!WAAnchorTag methodsFor: 'deprecated' stamp: 'jf 9/24/2008 10:59'!
mailto: aString
	self seasideDeprecatedApi: 'WAAnchorTag>>mailto:'
		details: 'Use WAAnchorTag>>url: passing in a "mailto:" URL.'.
	self url: 'mailto:' , aString; with: aString! !

!WAAnchorTag methodsFor: 'attributes' stamp: 'pmm 11/11/2008 22:56'!
name: aString
	"This attribute names the current anchor so that it may be the destination of another link. The value of this attribute must be a unique anchor name. The scope of this name is the current document. Note that this attribute shares the same name space as the id attribute."
	self attributes at: 'name' put: aString! !

!WAAnchorTag methodsFor: 'public' stamp: 'lr 10/28/2007 14:42'!
navigation
	"Makes the receiving anchor a purely navigational link, this is it won't redirect after processing the callbacks but directly process with the render phase. Don't use this feature if you change your model in the callback'."

	self url addParameter: '_n'! !

!WAAnchorTag methodsFor: 'deprecated' stamp: 'jf 9/24/2008 10:55'!
newTarget
	self seasideDeprecatedApi: 'WAAnchorTag>>newTarget'
		details: 'Use "target: ''_new''".'.
	self target: '_new'! !

!WAAnchorTag methodsFor: 'callbacks' stamp: 'lr 3/29/2008 16:39'!
on: aSymbol of: anObject
	self callback: (MessageSend receiver: anObject selector: aSymbol).
	self with: (self labelForSelector: aSymbol of: anObject)! !

!WAAnchorTag methodsFor: 'deprecated' stamp: 'jf 9/24/2008 10:54'!
rel: aString
	self seasideDeprecatedApi: 'WAAnchorTag>>rel:'
		details: 'Use WAAnchorTag>>relationship:.'.
	self relationship: aString! !

!WAAnchorTag methodsFor: 'attributes' stamp: 'lr 10/28/2007 14:42'!
relationship: aString
	self attributes at: 'rel' put: aString! !

!WAAnchorTag methodsFor: 'url' stamp: 'lr 8/17/2007 15:25'!
resourceUrl: aString
	self url: (canvas context absoluteUrlForResource: aString)! !

!WAAnchorTag methodsFor: 'convenience' stamp: 'lr 10/28/2007 14:42'!
submitFormNamed: aString
	"Submits a form with the id aString."

	self onClick: 'document.forms[' , aString printString , '].submit()'! !

!WAAnchorTag methodsFor: 'accessing' stamp: 'lr 8/17/2007 15:15'!
tag
	^ 'a'! !

!WAAnchorTag methodsFor: 'deprecated' stamp: 'jf 9/24/2008 10:53'!
target: aString 
	self seasideDeprecatedApi: 'WAAnchorTag>>target:'.
	self attributes 
		at: 'target'
		put: aString! !

!WAAnchorTag methodsFor: 'deprecated' stamp: 'jf 9/24/2008 10:52'!
text: aString
	self seasideDeprecatedApi: 'WAAnchorTag>>text:'
		details: 'Use WAAnchorTag>>with:.'.
	self with: aString! !

!WAAnchorTag methodsFor: 'accessing' stamp: 'lr 8/17/2007 15:17'!
url
	^ url ifNil: [ url := canvas context actionUrl copy ]! !

!WAAnchorTag methodsFor: 'accessing' stamp: 'lr 8/17/2007 15:06'!
url: aString
	url := aString! !

!WAAnchorTag methodsFor: 'public' stamp: 'lr 8/17/2007 15:11'!
with: aBlock
	url isNil
		ifFalse: [ self attributes at: 'href' put: url ]
		ifTrue: [
			((self attributes includesKey: 'href') or: [ self attributes includesKey: 'name' ])
				ifFalse: [ self attributes at: 'href' put: 'javascript:void(0)' ] ].
	super with: aBlock! !

WAAnchorTag subclass: #WAImageMapTag
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Seaside-Canvas-Tags'!

!WAImageMapTag commentStamp: 'lr 8/17/2007 18:56' prior: 0!
A WAImageMapTag is supposed to be used like this:

	html map
		callback: [ :point | self click: point ];
		with: [ html image url: 'foo.gif' ]
			
An example can be found in WAScreenshot.

Technincal:
http://www.w3.org/TR/html4/struct/objects.html#include-maps

The location clicked is passed to the server as follows. The user agent derives a new URI from the URI specified by the href attribute of the A element, by appending `?' followed by the x and y coordinates, separated by a comma. The link is then followed using the new URI. For instance, in the given example, if the user clicks at the location x=10, y=27 then the derived URI is "http://www.acme.com/cgi-bin/competition?10,27".
!

!WAImageMapTag methodsFor: 'callbacks' stamp: 'lr 3/24/2008 18:32'!
callback: aBlock
	"The parameter for the image-map callback must be the last one, as the web-browser will use the same name and replace it with the coordinates."

	aBlock fixCallbackTemps.
	self url
		addParameter: (canvas callbacks registerCallback: [ :value |
			aBlock value: (self parseImageMap: value) ])
		value: ''! !

!WAImageMapTag methodsFor: 'private' stamp: 'lr 3/24/2008 18:32'!
parseImageMap: aString
	| stream x y |
	stream := aString readStream.
	stream next = $? ifFalse: [ ^ nil ].
	x := Integer readFrom: stream.
	stream next = $, ifFalse: [ ^ nil ].
	y := Integer readFrom: stream.
	^ x @ y! !

WAAnchorTag subclass: #WAPopupAnchorTag
	instanceVariableNames: 'name features'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Seaside-Canvas-Tags'!

!WAPopupAnchorTag commentStamp: 'lr 7/4/2007 20:46' prior: 0!
I am used to create a link that opens in a new window. A name can be specified using #name: and many features can be configured. Note, that not all features are supported on all web browser platforms. If Javascript is disabled the link will behave like any other anchor.

Most of the time a popup-anchor is created like this:

	html popupAnchor
		callback: [ WARenderLoop new call: WACounter new ];
		with: 'Open the counter within a new window'

This code creates a new render-loop and displays a new instance of WACounter within the new browser window.		
!

!WAPopupAnchorTag methodsFor: 'features' stamp: 'lr 7/4/2007 20:32'!
dependent: aBoolean
	"Specifies whether the new window is closed as well when the parent window gets closed."

	features at: #dependent put: aBoolean! !

!WAPopupAnchorTag methodsFor: 'features' stamp: 'lr 10/28/2007 14:42'!
extent: aPoint
	"Specifies the width and height of the new window."

	self width: aPoint x; height: aPoint y! !

!WAPopupAnchorTag methodsFor: 'private' stamp: 'pmm 8/31/2008 14:45'!
featureString
	^ String streamContents: [ :stream | 
		features associations do: [ :association | 
			stream
				nextPutAll: association key;
				nextPut: $=.
			stream nextPutAll: (association value == true 
					ifTrue: [ 'yes' ]
					ifFalse: [
						association value == false 
							ifTrue: [ 'no' ]
							ifFalse: [ association value seasideString ] ]) ]
			separatedBy: [ stream nextPut: $, ] ]! !

!WAPopupAnchorTag methodsFor: 'features' stamp: 'lr 10/28/2007 14:42'!
height: anInteger
	"Specifies the height of the new window."

	features at: #height put: anInteger! !

!WAPopupAnchorTag methodsFor: 'initialize-release' stamp: 'lr 7/4/2007 20:16'!
initialize
	super initialize.
	name := '_blank'.
	features := Dictionary new! !

!WAPopupAnchorTag methodsFor: 'features' stamp: 'lr 10/28/2007 14:42'!
left: anInteger
	"Specifies the x coordinate of the top left corner of the new window. "

	features at: #left put: anInteger! !

!WAPopupAnchorTag methodsFor: 'features' stamp: 'lr 7/4/2007 20:24'!
location: aBoolean
	"Specifies whether to display the address line in the new window."

	features at: #location put: aBoolean! !

!WAPopupAnchorTag methodsFor: 'features' stamp: 'lr 7/4/2007 20:25'!
menubar: aBoolean
	"Specifies whether to display the browser menu bar."

	features at: #menubar put: aBoolean! !

!WAPopupAnchorTag methodsFor: 'accessing' stamp: 'lr 7/4/2007 20:22'!
name
	^ name! !

!WAPopupAnchorTag methodsFor: 'accessing' stamp: 'lr 7/4/2007 19:56'!
name: aString
	"String specifying the name of the new window. If a window with this name already exists, then the new content will be displayed in that existing window, rather than creating a new one."

	name := aString! !

!WAPopupAnchorTag methodsFor: 'features' stamp: 'lr 10/28/2007 14:42'!
position: aPoint
	"Specifies the x and y coordinates of the top left corner of the new window."

	self left: aPoint x; top: aPoint y! !

!WAPopupAnchorTag methodsFor: 'features' stamp: 'lr 7/4/2007 20:25'!
resizable: aBoolean
	"Specifies whether the new window is resizable."

	features at: #resizable put: aBoolean! !

!WAPopupAnchorTag methodsFor: 'features' stamp: 'lr 7/4/2007 20:25'!
scrollbars: aBoolean
	"Specifies whether the new window should have scrollbars."

	features at: #scrollbars put: aBoolean! !

!WAPopupAnchorTag methodsFor: 'features' stamp: 'lr 7/4/2007 20:24'!
status: aBoolean
	"Specifies whether to display the browser status bar."

	features at: #status put: aBoolean! !

!WAPopupAnchorTag methodsFor: 'features' stamp: 'lr 7/4/2007 20:24'!
toolbar: aBoolean
	"Specifies whether to display the toolbar in the new window."

	features at: #toolbar put: aBoolean! !

!WAPopupAnchorTag methodsFor: 'features' stamp: 'lr 10/28/2007 14:42'!
top: anInteger
	"Specifies the y coordinate of the top left corner of the new window. "

	features at: #top put: anInteger! !

!WAPopupAnchorTag methodsFor: 'features' stamp: 'lr 10/28/2007 14:42'!
width: anInteger
	"Specifies the width of the new window."

	features at: #width put: anInteger! !

!WAPopupAnchorTag methodsFor: 'public' stamp: 'lr 7/4/2007 20:33'!
with: aBlock
	self onClick: 'window.open(this.href,' , self name printString , ',' , self featureString printString , ');return false'.
	super with: aBlock! !

WATagBrush subclass: #WABasicFormTag
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Seaside-Canvas-Tags'!

!WABasicFormTag commentStamp: 'MAD 8/30/2008 15:21' prior: 0!
Acts as a basic form, with no defaultAction, and no holding of _s and _c values.
!

!WABasicFormTag methodsFor: 'attributes' stamp: 'lr 3/24/2008 13:01'!
acceptCharset: aString 
	"This attribute specifies the list of character encodings for input data that is accepted by the server processing this form. The value is a space- and/or comma-delimited list of charset values. The client must interpret this list as an exclusive-or list, i.e., the server is able to accept any single character encoding per entity received.

The default value for this attribute is the reserved string 'UNKNOWN'. User agents may interpret this value as the character encoding that was used to transmit the document containing this FORM element."
	self attributes 
		at: 'accept-charset'
		put: aString! !

!WABasicFormTag methodsFor: 'attributes' stamp: 'lr 3/24/2008 13:01'!
action: aString 
	"This attribute specifies a form processing agent. User agent behavior for a value other than an HTTP URI is undefined."
	self attributes 
		at: 'action'
		put: aString! !

!WABasicFormTag methodsFor: 'private' stamp: 'MAD 8/30/2008 14:55'!
defaultUrlCharset
	"the character set used for POSTs that are application/x-www-form-urlencoded
	and GETs
	There are standards that dictate this:
	http://www.w3.org/International/O-URL-code.html
	but that doesn't mean is will work with your favorite old browser"
	^ 'utf-8'! !

!WABasicFormTag methodsFor: 'attributes' stamp: 'pmm 5/9/2008 19:27'!
get
	"tells the user agent to use the HTTP POST method to submit this form"
	self method: 'get'! !

!WABasicFormTag methodsFor: 'initialize-release' stamp: 'MAD 8/30/2008 14:55'!
initialize
	super initialize.
	self acceptCharset: self defaultUrlCharset! !

!WABasicFormTag methodsFor: 'attributes' stamp: 'lr 3/24/2008 13:01'!
method: aString 
	"This attribute specifies which HTTP method will be used to submit the form data set. Possible (case-insensitive) values are 'get' (the default) and 'post'."
	self attributes 
		at: 'method'
		put: aString! !

!WABasicFormTag methodsFor: 'attributes' stamp: 'jf 9/11/2008 15:07'!
multipart
	"Sets the content type used to submit the form to the server (when the value of method is 'post') to multipart/form-data instead the default application/x-www-form-urlencoded. This should be used in combination with a WAFileUploadTag."
	self attributes 
		at: 'enctype'
		put: 'multipart/form-data'.
		
	"set the request encoding to the body encoding"
	self acceptCharset: self application charSet! !

!WABasicFormTag methodsFor: 'attributes' stamp: 'lr 10/28/2007 14:42'!
multipart: aBoolean
	"Sets the content type used to submit the form to the server.
	true: sets the content type to multipart/form-data
	false: leavess the content type to default application/x-www-form-urlencoded"
	aBoolean ifTrue: [ self multipart ]! !

!WABasicFormTag methodsFor: 'attributes' stamp: 'pmm 11/11/2008 22:58'!
name: aString
	"This attribute names the element so that it may be referred to from style sheets or scripts. Note. This attribute has been included for backwards compatibility. Applications should use the id attribute to identify elements."
	self attributes at: 'name' put: aString! !

!WABasicFormTag methodsFor: 'attributes' stamp: 'lr 3/24/2008 13:01'!
noAutocomplete
	self attributes 
		at: 'autocomplete'
		put: 'off'! !

!WABasicFormTag methodsFor: 'attributes' stamp: 'pmm 5/9/2008 19:27'!
post
	"tells the user agent to use the HTTP POST method to submit this form"
	self method: 'post'! !

!WABasicFormTag methodsFor: 'initialize-release' stamp: 'pmm 6/21/2007 20:57'!
setParent: aBrush canvas: aCanvas
	super setParent: aBrush canvas: aCanvas.
	self post; action: aCanvas context actionUrl withoutParameters! !

!WABasicFormTag methodsFor: 'accessing' stamp: 'avi 6/17/2004 11:29'!
tag
	^ 'form'! !

WABasicFormTag subclass: #WAFormTag
	instanceVariableNames: 'defaultAction'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Seaside-Canvas-Tags'!

!WAFormTag commentStamp: 'pmm 9/15/2007 08:23' prior: 0!
The FORM element acts as a container for input elements and buttons.

Evaluation order:
The input fields callbacks will be evaluated in the order they
appear in the XHTML. Buttons will always be evaluated last, no matter
where they are positioned.!

!WAFormTag methodsFor: 'private' stamp: 'MAD 8/30/2008 12:24'!
after
	canvas div: [
		canvas context actionUrl parameters keysAndValuesDo: [ :k :v |
			canvas hiddenInput
				name: k;
				value: v ] ]! !

!WAFormTag methodsFor: 'private' stamp: 'MAD 8/30/2008 12:24'!
before
	"Define the default action form buttons. Some implementation notes on this feature: (1) a tab-index of -1 is not valid XHTML, but most todays browser accept it and ignore the element in the tab-order. (2) Internet Explorer requires an additional text field (without other functionality) to make the default action work. Other browser should not include this text-field, as it prevents remembering form input."

	defaultAction ifNil: [ ^ nil ].
	canvas div: [
		canvas submitButton
			tabIndex: -1;
			value: 'Default';
			callback: defaultAction;
			style: 'position: absolute; top: -100em'.
		self isInternetExplorer ifTrue: [
			canvas textInput
				tabIndex: -1;
				callback: [ :v | ];
				style: 'position: absolute; top: -100em' ] ]! !

!WAFormTag methodsFor: 'callbacks' stamp: 'MAD 8/30/2008 13:05'!
defaultAction: aBlock
	"The default action gets evaluated whenever the user presses submits
	the form by pressing enter without having the focus on a specific
	submit-button."
	defaultAction := aBlock! !

!WAFormTag methodsFor: 'private' stamp: 'pmm 8/30/2008 15:11'!
isInternetExplorer
	"Tries to find out whether the current request is made by IE. This is extremely unrelyable."
	| userAgent |
	userAgent := self requestContext request userAgent.
	^ userAgent notNil and: 
		[ (userAgent includesSubString: 'MSIE') and: [ (userAgent includesSubString: 'Opera') not ] ]! !

WATagBrush subclass: #WABreakTag
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Seaside-Canvas-Tags'!

!WABreakTag commentStamp: 'pmm 9/22/2007 22:22' prior: 0!
The BR element forcibly breaks (ends) the current line of text. Authors are advised to use style sheets to control text flow around floating images and other objects.

Prohibiting a line break 

Sometimes authors may want to prevent a line break from occurring between two words. The &nbsp; entity (&#160; or &#xA0;) acts as a space where user agents should not cause a line break.!

!WABreakTag methodsFor: 'testing' stamp: 'lr 3/26/2007 21:10'!
isClosed
	^ true! !

!WABreakTag methodsFor: 'accessing' stamp: 'lr 3/26/2007 21:10'!
tag
	^ 'br'! !

WATagBrush subclass: #WACollectionTag
	instanceVariableNames: 'list selected callbackBlock labelBlock'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Seaside-Canvas-Tags'!

!WACollectionTag commentStamp: 'pmm 4/23/2007 20:59' prior: 0!
This element allows the use to select one (in single selection mode) or multiple (in single selection mode) elements. Multiple selection can be triggered with #beMultiple, single selection can be triggered with #beSingle .Default is single selection.

The general idea is that you pass the colletion of options to #list: and use #labels: to customize their rendering. The selected option(s) can be set with: #selected:.


This whole class is kind of an akward hack, but noone needs MI anyway. You can always get a way with composition and delegation!

!WACollectionTag methodsFor: 'adding' stamp: 'lr 6/6/2007 13:22'!
add: anObject
	self list add: anObject! !

!WACollectionTag methodsFor: 'adding' stamp: 'lr 6/6/2007 13:22'!
addAll: aCollection
	self list addAll: aCollection! !

!WACollectionTag methodsFor: 'private' stamp: 'lr 10/28/2007 14:42'!
before
	super before.
	self hasList ifTrue: [
		self list do: [ :each |
			self
				renderListItem: each
				labelled: (self labelForOption: each) ] ]! !

!WACollectionTag methodsFor: 'callbacks' stamp: 'lr 6/7/2007 07:48'!
callback: aBlock
	callbackBlock := aBlock fixCallbackTemps! !

!WACollectionTag methodsFor: 'testing' stamp: 'lr 6/7/2007 07:46'!
hasCallback
	^ callbackBlock notNil! !

!WACollectionTag methodsFor: 'testing' stamp: 'lr 6/6/2007 13:30'!
hasList
	^ list notNil and: [ list notEmpty ]! !

!WACollectionTag methodsFor: 'testing' stamp: 'lr 10/28/2007 14:42'!
isSelected: anObject
	"Test if anObject is currently selected."

 	^ selected = anObject! !

!WACollectionTag methodsFor: 'private' stamp: 'lr 6/6/2007 13:53'!
labelForOption: anObject
	^ labelBlock isNil
		ifTrue: [ anObject ]
		ifFalse: [ labelBlock value: anObject ]! !

!WACollectionTag methodsFor: 'callbacks' stamp: 'lr 10/28/2007 14:42'!
labels: aBlock
	"Allows to customize the rendering of list items by passing a one argument block that converts each option to a string. If you need to do custom html rendering for the options use #with: or override #renderOn: in your objects."

	labelBlock := aBlock fixCallbackTemps! !

!WACollectionTag methodsFor: 'accessing' stamp: 'lr 6/6/2007 13:22'!
list
	^ list ifNil: [ list := OrderedCollection new ]! !

!WACollectionTag methodsFor: 'accessing' stamp: 'lr 10/28/2007 14:42'!
list: aCollection
	"Append aCollection of items to display. The rendering can be customized using #labels:."

	aCollection ifNotNil: [ self addAll: aCollection ]! !

!WACollectionTag methodsFor: 'callbacks' stamp: 'lr 3/29/2008 16:39'!
on: aSelector of: anObject
	self selected: (anObject perform: aSelector).
	self callback: (MessageSend receiver: anObject selector: aSelector asMutator)! !

!WACollectionTag methodsFor: 'private' stamp: 'lr 6/7/2007 07:50'!
performCallback: anObject
	self hasCallback ifTrue: [ callbackBlock value: anObject ]! !

!WACollectionTag methodsFor: 'private' stamp: 'lr 6/7/2007 07:56'!
renderListItem: anObject labelled: aString
	self subclassResponsibility! !

!WACollectionTag methodsFor: 'accessing' stamp: 'lr 10/28/2007 14:42'!
selected: anObject
	"Set anObject to be selected."

	selected := anObject! !

WACollectionTag subclass: #WAListTag
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Seaside-Canvas-Tags'!

!WAListTag commentStamp: 'pmm 6/30/2007 00:46' prior: 0!
Abstract base class for ordered and unordered list tags.

Ordered and unordered lists are rendered in an identical manner except that visual user agents number ordered list items. User agents may present those numbers in a variety of ways. Unordered list items are not numbered.

Both types of lists are made up of sequences of list items defined by the LI element (whose end tag may be omitted).

Lists may also be nested.!

!WAListTag methodsFor: 'private' stamp: 'lr 6/7/2007 08:01'!
renderListItem: anObject labelled: aString
	canvas listItem
		class: 'option-selected' if: (self isSelected: anObject);
		with: [
			self hasCallback
				ifFalse: [ canvas render: aString ]
				ifTrue: [
					canvas anchor
						callback: [ self performCallback: anObject ];
						with: aString ] ]! !

WAListTag subclass: #WAOrderedListTag
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Seaside-Canvas-Tags'!

!WAOrderedListTag commentStamp: 'pmm 6/30/2007 00:47' prior: 0!
See superclass comment.!

!WAOrderedListTag methodsFor: 'accessing' stamp: 'lr 4/9/2007 17:12'!
tag
	^ 'ol'! !

WAListTag subclass: #WAUnorderedListTag
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Seaside-Canvas-Tags'!

!WAUnorderedListTag commentStamp: 'pmm 6/30/2007 00:47' prior: 0!
See superclass comment.!

!WAUnorderedListTag methodsFor: 'accessing' stamp: 'lr 4/9/2007 17:12'!
tag
	^ 'ul'! !

WACollectionTag subclass: #WASelectTag
	instanceVariableNames: 'enabledBlock isOptional optionalLabel'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Seaside-Canvas-Tags'!

!WASelectTag commentStamp: 'pmm 4/24/2007 11:12' prior: 0!
This element allows the use to select one (in single selection mode) or multiple (in single selection mode) elements. Default is single selection. Multiple selection can be triggered with #beMultiple.

Single selection is in general a drop-down list, so make sure the list of options is not too big.
Multiple selection has in general crappy browser support and a list of checkboxes is in general the better option.

If in single selection mode and you want enable "no selection" see #beOptional.

If you absolutely need to you can render the options yourself with 'html option' inside #with:

Make sure to check the superclass for more methods.

See WAInputTest >> #renderSingleSelectionOn: and WAInputTest >> #renderMultiSelectionOn: for examples.!

WASelectTag subclass: #WAMultiSelectTag
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Seaside-Canvas-Tags'!

!WAMultiSelectTag commentStamp: 'obi 4/2/2008 22:54' prior: 0!
See superclass.!

!WAMultiSelectTag methodsFor: 'private' stamp: 'lr 10/28/2007 14:42'!
closeTag
	super closeTag.
	self hasCallback ifFalse: [ ^ self ].
	canvas insert: [
		canvas hiddenInput callback: [
			self hasCallback
				ifTrue: [ callbackBlock value: selected ] ] ]! !

!WAMultiSelectTag methodsFor: 'private' stamp: 'lr 6/6/2007 16:18'!
dispatchCallback
	^ canvas callbacks registerMultiDispatchCallback! !

!WAMultiSelectTag methodsFor: 'testing' stamp: 'lr 6/6/2007 16:14'!
isSelected: anObject
	^ selected notNil and: [ selected includes: anObject ]! !

!WAMultiSelectTag methodsFor: 'private' stamp: 'lr 10/28/2007 14:42'!
openTag
	self hasCallback ifTrue: [
		canvas insert: [
			canvas hiddenInput
				callback: [ selected := OrderedCollection new ] ] ].
	self attributes at: 'multiple' put: true.
	super openTag! !

!WAMultiSelectTag methodsFor: 'private' stamp: 'lr 6/6/2007 18:37'!
performCallback: anObject
	selected add: anObject! !

!WASelectTag methodsFor: 'attributes' stamp: 'lr 10/28/2007 14:42'!
beOptional
	"This adds a nil item to #list: which has the semantic of no selection. #optionalLabel: is the label for nil. This only really makes sense in single selection mode."

	isOptional := true! !

!WASelectTag methodsFor: 'attributes' stamp: 'lr 10/28/2007 14:42'!
beSubmitOnChange
	"Submit the form in the user selects a value."

	self onChange: 'submit()'! !

!WASelectTag methodsFor: 'private' stamp: 'lr 6/6/2007 17:44'!
before
	self isOptional ifTrue: [ self renderOptional ].
	super before! !

!WASelectTag methodsFor: 'private' stamp: 'lr 6/6/2007 16:17'!
dispatchCallback
	^ canvas callbacks registerDispatchCallback! !

!WASelectTag methodsFor: 'callbacks' stamp: 'lr 6/6/2007 13:38'!
enabled: aBlock
	"Enable only those elements for selection for whom aBlock return true."

	enabledBlock := aBlock fixCallbackTemps! !

!WASelectTag methodsFor: 'initialize-release' stamp: 'lr 6/6/2007 17:43'!
initialize
	super initialize.
	isOptional := false! !

!WASelectTag methodsFor: 'testing' stamp: 'lr 6/6/2007 16:21'!
isEnabled: anObject
	"Test if anObject is enabled."

 	^ enabledBlock isNil or: [ enabledBlock value: anObject ]! !

!WASelectTag methodsFor: 'testing' stamp: 'lr 6/6/2007 17:42'!
isOptional
	^ isOptional! !

!WASelectTag methodsFor: 'attributes' stamp: 'lr 10/28/2007 14:42'!
name: aString
	"This attribute assigns the control name."

	self attributes at: 'name' put: aString! !

!WASelectTag methodsFor: 'private' stamp: 'lr 6/6/2007 16:23'!
openTag
	self attributes at: 'name' ifAbsentPut: [ self dispatchCallback ].
	super openTag! !

!WASelectTag methodsFor: 'attributes' stamp: 'lr 10/28/2007 14:42'!
optionalLabel: aString
	"The label for the nil element. See #beOptional."

	optionalLabel := aString! !

!WASelectTag methodsFor: 'private' stamp: 'lr 6/7/2007 07:57'!
renderListItem: anObject labelled: aString
	| option |
	option := canvas option.
	self hasCallback
		ifTrue: [ option callback: [ self performCallback: anObject ] ].
	option
		selected: (self isSelected: anObject);
		disabled: (self isEnabled: anObject) not;
		with: aString! !

!WASelectTag methodsFor: 'private' stamp: 'lr 6/7/2007 07:58'!
renderOptional
	self renderListItem: nil labelled: optionalLabel! !

!WASelectTag methodsFor: 'attributes' stamp: 'lr 10/28/2007 14:42'!
size: aNumber
	"If a SELECT element is presented as a scrolled list box, this attribute specifies the number of rows in the list that should be visible at the same time. Visual user agents are not required to present a SELECT element as a list box; they may use any other mechanism, such as a drop-down menu."

	self attributes at: 'size' put: aNumber! !

!WASelectTag methodsFor: 'accessing' stamp: 'ac 3/17/2005 16:44'!
tag
	^ 'select'! !

WATagBrush subclass: #WADivTag
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Seaside-Canvas-Tags'!

!WADivTag commentStamp: 'obi 4/2/2008 22:48' prior: 0!
The <div> tag defines a division/section in a document.

Use the <div> tag to group block-elements to format them with styles.
!

!WADivTag methodsFor: 'public' stamp: 'avi 6/3/2005 15:40'!
clear
	self class: 'clear'.
	self with: [canvas space]! !

!WADivTag methodsFor: 'accessing' stamp: 'avi 6/3/2005 15:39'!
tag
	^ 'div'! !

WATagBrush subclass: #WAFieldSetTag
	instanceVariableNames: 'legend'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Seaside-Canvas-Tags'!

!WAFieldSetTag commentStamp: 'pmm 5/10/2007 10:12' prior: 0!
The FIELDSET element allows authors to group thematically related controls and labels. Grouping controls makes it easier for users to understand their purpose while simultaneously facilitating tabbing navigation for visual user agents and speech navigation for speech-oriented user agents. The proper use of this element makes documents more accessible.

The LEGEND element allows authors to assign a caption to a FIELDSET. The legend improves accessibility when the FIELDSET is rendered non-visually.!

!WAFieldSetTag methodsFor: 'private' stamp: 'lr 10/28/2007 14:42'!
before
	legend ifNotNil: [canvas legend: legend]! !

!WAFieldSetTag methodsFor: 'accessing' stamp: 'lr 10/28/2007 14:42'!
legend: aString
	legend := aString! !

!WAFieldSetTag methodsFor: 'accessing' stamp: 'mb 3/8/2006 17:44'!
tag
	^ 'fieldset'! !

WATagBrush subclass: #WAFormInputTag
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Seaside-Canvas-Tags'!

!WAFormInputTag commentStamp: 'obi 4/2/2008 22:49' prior: 0!
The <input> tag defines the start of an input field where the user can enter data.!

WAFormInputTag subclass: #WAAbstractTextAreaTag
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Seaside-Canvas-Tags'!

!WAAbstractTextAreaTag commentStamp: 'pmm 4/23/2007 21:10' prior: 0!
An abstract class to describe the HTML elements that allow the user to enter text.!

!WAAbstractTextAreaTag methodsFor: 'callbacks' stamp: 'lr 2/25/2007 23:11'!
callback: aBlock
	self name: (canvas callbacks registerCallback: aBlock).
	self value: self value! !

!WAAbstractTextAreaTag methodsFor: 'javascript' stamp: 'lr 6/24/2007 10:03'!
exampleText: aString
	(self value isEmptyOrNil or: [ self value = aString ]) ifFalse: [ ^ self ].
	self onFocus: 'if(value==' , aString printString , '){value='''';style.color=null}'.
	self style: 'color: #aaa'; value: aString! !

!WAAbstractTextAreaTag methodsFor: 'javascript' stamp: 'lr 10/28/2007 14:42'!
setCursorPosition: anInteger
	self setSelectionFrom: anInteger to: anInteger! !

!WAAbstractTextAreaTag methodsFor: 'javascript' stamp: 'lr 4/9/2008 10:49'!
setSelectionFrom: aStartInteger to: aStopInteger
	"Insert the javascript for setting the selection"
	| aStream |
	self ensureId.
	aStream := (String new: 30) writeStream.
	aStream
		nextPutAll: 'setSelection(''';
		nextPutAll: self id;
		nextPutAll: ''''.
	aStartInteger isNumber
		ifTrue: [aStream nextPut: $,; print: aStartInteger]
		ifFalse: [aStream nextPutAll: ',1'].
	aStopInteger isNumber
		ifTrue: [aStream nextPut: $,; print: aStopInteger ].
	aStream nextPut: $).
	self requestContext addLoadScript: aStream contents! !

WAAbstractTextAreaTag subclass: #WATextAreaTag
	instanceVariableNames: 'value'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Seaside-Canvas-Tags'!

!WATextAreaTag commentStamp: 'pmm 4/24/2007 11:13' prior: 0!
I am a multi line text input widget. See WAInputTest >> #renderTextAreaWithExampleOn: for examples.!

!WATextAreaTag methodsFor: 'attributes' stamp: 'lr 3/24/2008 13:01'!
columns: anInteger 
	"This attribute specifies the visible width in average character widths. User agents may wrap visible text lines to keep long lines visible without the need for scrolling."
	self attributes 
		at: 'cols'
		put: anInteger! !

!WATextAreaTag methodsFor: 'initialize-release' stamp: 'lr 2/16/2008 18:42'!
initialize
	super initialize.
	self rows: 'auto'; columns: 'auto'! !

!WATextAreaTag methodsFor: 'testing' stamp: 'lr 3/26/2007 21:12'!
isClosed
	^ false! !

!WATextAreaTag methodsFor: 'attributes' stamp: 'lr 3/24/2008 13:01'!
rows: anInteger 
	"This attribute specifies the number of visible text lines. If more are entered, the widget scrolls."
	self attributes 
		at: 'rows'
		put: anInteger! !

!WATextAreaTag methodsFor: 'accessing' stamp: 'ac 4/13/2005 01:51'!
tag
	^ 'textarea'! !

!WATextAreaTag methodsFor: 'accessing' stamp: 'lr 9/4/2007 15:53'!
value
	^ value! !

!WATextAreaTag methodsFor: 'accessing' stamp: 'ac 8/30/2005 16:26'!
value: aString
	value := aString! !

!WATextAreaTag methodsFor: 'public' stamp: 'lr 9/4/2007 15:57'!
with: aBlock
	super with: (value ifNil: [ aBlock ])! !

WAAbstractTextAreaTag subclass: #WATextInputTag
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Seaside-Canvas-Tags'!

!WATextInputTag commentStamp: 'pmm 4/24/2007 11:13' prior: 0!
I am a single line text input widget. See WAInputTest >> #renderTextInputWithExampleOn: for examples.!

WATextInputTag subclass: #WAPasswordInputTag
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Seaside-Canvas-Tags'!

!WAPasswordInputTag commentStamp: 'pmm 4/23/2007 21:12' prior: 0!
I am a password input widget that does not display the entered text.!

!WAPasswordInputTag methodsFor: 'accessing' stamp: 'lr 9/23/2005 17:33'!
type
	^ 'password'! !

!WATextInputTag methodsFor: 'attributes' stamp: 'lr 3/24/2008 13:01'!
maxLength: aNumber 
	"This attribute specifies the maximum number of characters the user may enter. This number may exceed the specified #size:, in which case the user agent should offer a scrolling mechanism. The default value for this attribute is an unlimited number."
	self attributes 
		at: 'maxlength'
		put: aNumber! !

!WATextInputTag methodsFor: 'attributes' stamp: 'lr 3/24/2008 13:01'!
size: aNumber 
	"This attribute tells the user agent the initial width of the widget. The width is given in number of characters."
	self attributes 
		at: 'size'
		put: aNumber! !

!WATextInputTag methodsFor: 'accessing' stamp: 'lr 9/23/2005 13:50'!
type
	^ 'text'! !

!WATextInputTag methodsFor: 'public' stamp: 'lr 8/30/2008 16:22'!
with: anObject 
	anObject isNil ifFalse: [ self value: anObject seasideString ].
	super with: nil! !

WAFormInputTag subclass: #WAButtonTag
	instanceVariableNames: 'type value'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Seaside-Canvas-Tags'!

!WAButtonTag commentStamp: 'pmm 7/28/2007 08:54' prior: 0!
Buttons created with the BUTTON element function just like buttons created with the INPUT element, but they offer richer rendering possibilities: the BUTTON element may have content. For example, a BUTTON element that contains an image functions like and may resemble an INPUT element whose type is set to "image", but the BUTTON element type allows content.!

!WAButtonTag methodsFor: 'accessing' stamp: 'pmm 7/28/2007 08:40'!
bePush
	"Creates a push button. Push buttons have no default behavior. Each push button may have client-side scripts associated with the element's event attributes. When an event occurs (e.g., the user presses the button, releases it, etc.), the associated script is triggered."
	type := 'button'! !

!WAButtonTag methodsFor: 'accessing' stamp: 'pmm 7/28/2007 08:34'!
beReset
	"Creates a reset button. When activated it resets all controls to their initial values."
	type := 'reset'! !

!WAButtonTag methodsFor: 'accessing' stamp: 'pmm 7/28/2007 08:33'!
beSubmit
	"Creates a submit button. When activated, a submit button submits a form. A form may contain more than one submit button. This is the default."
	type := 'submit'! !

!WAButtonTag methodsFor: 'callbacks' stamp: 'lr 4/11/2008 08:46'!
callback: aBlock
	self name: (canvas callbacks registerActionCallback: aBlock)! !

!WAButtonTag methodsFor: 'testing' stamp: 'lr 4/11/2008 08:48'!
canHaveShortcut
	^ true! !

!WAButtonTag methodsFor: 'initialize-release' stamp: 'pmm 7/28/2007 08:34'!
initialize
	super initialize.
	self beSubmit! !

!WAButtonTag methodsFor: 'testing' stamp: 'lr 6/13/2007 21:13'!
isClosed
	^ false! !

!WAButtonTag methodsFor: 'callbacks' stamp: 'lr 4/11/2008 08:46'!
on: aSymbol of: anObject
	self value: (self labelForSelector: aSymbol of: anObject).
	self callback: (MessageSend receiver: anObject selector: aSymbol)! !

!WAButtonTag methodsFor: 'accessing' stamp: 'lr 6/13/2007 21:13'!
tag
	^ 'button'! !

!WAButtonTag methodsFor: 'accessing' stamp: 'lr 8/8/2007 14:48'!
type
	^ type! !

!WAButtonTag methodsFor: 'accessing' stamp: 'lr 9/18/2007 08:18'!
value
	^ value! !

!WAButtonTag methodsFor: 'accessing' stamp: 'lr 9/18/2007 08:19'!
value: aString
	value := aString! !

!WAButtonTag methodsFor: 'public' stamp: 'lr 9/18/2007 08:19'!
with: aBlock
	super with: (value ifNil: [ aBlock ])! !

WAFormInputTag subclass: #WACheckboxTag
	instanceVariableNames: 'value callback'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Seaside-Canvas-Tags'!

!WACheckboxTag commentStamp: 'pmm 6/29/2008 17:06' prior: 0!
The checkbox element is used with an associated property of typ boolean. It renders a HTML <input> element of type checkbox.

If this input element is disabled then the callback will still be evaluted but with an argument that is false. See method comment of #callback: for more information.

!

!WACheckboxTag methodsFor: 'private' stamp: 'jf 7/20/2008 09:53'!
after
	callback ifNotNil: [
		canvas hiddenInput callback: [
			callback value: value.
			value := false ] ]! !

!WACheckboxTag methodsFor: 'callbacks' stamp: 'pmm 6/2/2008 08:26'!
callback: aBlock
	"Compared to other form elements this input element is disabled then the callback will still be evaluted but with an argument that is false. So either don't assign a callback to disabled checkboxes or make the callback code aware that he might be invoked in a disabled state.
	
	However if you enable a disabled checkbox with JavaScript then the callback is invoked with the correct argument.
	
	http://code.google.com/p/seaside/issues/detail?id=18"
	value := false.
	callback := aBlock fixCallbackTemps.
	super callback: [value := true]! !

!WACheckboxTag methodsFor: 'testing' stamp: 'lr 4/11/2008 08:48'!
canHaveShortcut
	^ true! !

!WACheckboxTag methodsFor: 'attributes' stamp: 'lr 10/28/2007 14:42'!
onTrue: trueBlock onFalse: falseBlock
	trueBlock fixCallbackTemps. falseBlock fixCallbackTemps.
	self callback: [ :v | v ifTrue: [ trueBlock value ] ifFalse: [ falseBlock value ] ]! !

!WACheckboxTag methodsFor: 'convenience' stamp: 'lr 8/30/2008 16:22'!
submitFormNamed: formName 
	self onClick: 'submitForm(''' , formName seasideString , '''); return false;'! !

!WACheckboxTag methodsFor: 'accessing' stamp: 'lr 9/23/2005 13:44'!
type
	^ 'checkbox'! !

!WACheckboxTag methodsFor: 'attributes' stamp: 'lr 3/24/2008 13:01'!
value: aBoolean 
	self attributes 
		at: 'checked'
		put: aBoolean! !

WAFormInputTag subclass: #WAFileUploadTag
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Seaside-Canvas-Tags'!

!WAFileUploadTag commentStamp: 'pmm 4/15/2007 22:22' prior: 0!
I represent a file input element (<input type="file"). My callbacks will be evaluated with an instance of WAFile as an argument.!

!WAFileUploadTag methodsFor: 'attributes' stamp: 'pmm 5/2/2007 21:50'!
accept: aString
	"This attribute specifies a comma-separated list of content types that a server processing this form will handle correctly. User agents may use this information to filter out non-conforming files when prompting a user to select files to be sent to the server"
	self attributes at: 'accept' put: aString! !

!WAFileUploadTag methodsFor: 'callbacks' stamp: 'lr 10/21/2008 20:53'!
callback: aBlock
	self name: (canvas callbacks registerUploadCallback: aBlock)! !

!WAFileUploadTag methodsFor: 'callbacks' stamp: 'lr 3/29/2008 16:41'!
on: aSelector of: anObject
	self callback: (MessageSend receiver: anObject selector: aSelector asMutator)! !

!WAFileUploadTag methodsFor: 'accessing' stamp: 'lr 9/23/2005 13:45'!
type
	^ 'file'! !

!WAFormInputTag methodsFor: 'callbacks' stamp: 'lr 1/4/2007 09:35'!
callback: aBlock
	self name: (canvas callbacks registerCallback: aBlock)! !

!WAFormInputTag methodsFor: 'testing' stamp: 'lr 3/26/2007 21:12'!
isClosed
	^ true! !

!WAFormInputTag methodsFor: 'attributes' stamp: 'pmm 11/11/2008 23:00'!
name: aString
	"This attribute assigns the control name.
	
	A control's 'control name' is given by its name attribute. The scope of the name attribute for a control within a FORM element is the FORM element."
	self attributes at: 'name' put: aString! !

!WAFormInputTag methodsFor: 'callbacks' stamp: 'lr 3/29/2008 16:41'!
on: aSelector of: anObject
	self value: (anObject perform: aSelector).
	self callback: (MessageSend receiver: anObject selector: aSelector asMutator)! !

!WAFormInputTag methodsFor: 'attributes' stamp: 'pmm 4/23/2007 21:24'!
readonly: aBoolean
	"When set to true, this boolean attribute prohibits changes to the widget."
	self attributes at: 'readonly' put: aBoolean! !

!WAFormInputTag methodsFor: 'javascript' stamp: 'lr 4/9/2008 10:51'!
setFocus
	"Makes this element have the initial focus once the page is loaded aka autofocus."
	self ensureId.
	self requestContext addLoadScript: 'setFocus(' , self id printString , ')'! !

!WAFormInputTag methodsFor: 'attributes' stamp: 'lr 1/4/2007 09:34'!
submitOnClick
	self onClick: 'submit()'! !

!WAFormInputTag methodsFor: 'accessing' stamp: 'ac 3/16/2005 12:08'!
tag
	^ 'input'! !

!WAFormInputTag methodsFor: 'attributes' stamp: 'lr 1/4/2007 09:37'!
text: aString
	self value: aString! !

!WAFormInputTag methodsFor: 'accessing' stamp: 'lr 9/23/2005 13:43'!
type
	^ nil! !

!WAFormInputTag methodsFor: 'attributes' stamp: 'lr 1/4/2007 09:34'!
type: aString
	self attributes at: 'type' put: aString! !

!WAFormInputTag methodsFor: 'attributes' stamp: 'lr 1/4/2007 09:34'!
value
	^ self attributes at: 'value'! !

!WAFormInputTag methodsFor: 'attributes' stamp: 'lr 1/4/2007 09:34'!
value: anObject
	anObject ifNotNil: [ self attributes at: 'value' put: anObject ]! !

!WAFormInputTag methodsFor: 'public' stamp: 'lr 11/16/2006 15:31'!
with: aBlock
	self type isNil ifFalse: [
		self attributes at: 'type' ifAbsentPut: [ self type ].
		self class: self type ].
	super with: aBlock! !

WAFormInputTag subclass: #WAHiddenInputTag
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Seaside-Canvas-Tags'!

!WAHiddenInputTag commentStamp: 'pmm 9/22/2007 22:23' prior: 0!
Authors may create controls that are not rendered but whose values are submitted with a form. Authors generally use this control type to store information between client/server exchanges that would otherwise be lost due to the stateless nature of HTTP (see [RFC2616]). The INPUT element is used to create a hidden control.!

!WAHiddenInputTag methodsFor: 'accessing' stamp: 'lr 9/23/2005 17:45'!
type
	^ 'hidden'! !

WAFormInputTag subclass: #WARadioButtonTag
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Seaside-Canvas-Tags'!

!WARadioButtonTag commentStamp: 'pmm 6/1/2008 00:45' prior: 0!
Radio buttons works like checkboxes. The difference is that radio buttons, when sharing a single control name, are exclusive. This means that only one of those radio buttons can be checked, and when it's checked, the rest are automatically unchecked.

Radio buttons must be created through a radio group.

Example:
| group |
group := html radioGroup.
group radioButton
	selected: aBoolean;
	callback: [ self someThing ].!

!WARadioButtonTag methodsFor: 'attributes' stamp: 'lr 3/24/2008 13:01'!
beChecked
	self attributes 
		at: 'checked'
		put: true! !

!WARadioButtonTag methodsFor: 'attributes' stamp: 'lr 3/24/2008 13:01'!
beUnchecked
	self attributes 
		at: 'checked'
		put: false! !

!WARadioButtonTag methodsFor: 'callbacks' stamp: 'pmm 1/8/2007 12:14'!
callback: aBlock
	self value: (canvas callbacks registerCallback: aBlock)! !

!WARadioButtonTag methodsFor: 'testing' stamp: 'lr 4/11/2008 08:48'!
canHaveShortcut
	^ true! !

!WARadioButtonTag methodsFor: 'attributes' stamp: 'lr 10/28/2007 14:42'!
group: aRadioGroup
	self name: aRadioGroup key! !

!WARadioButtonTag methodsFor: 'attributes' stamp: 'lr 3/24/2008 13:01'!
selected: aBoolean 
	self attributes 
		at: 'checked'
		put: aBoolean! !

!WARadioButtonTag methodsFor: 'accessing' stamp: 'lr 9/23/2005 13:46'!
type
	^ 'radio'! !

WAFormInputTag subclass: #WASubmitButtonTag
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Seaside-Canvas-Tags'!

!WASubmitButtonTag commentStamp: 'obi 4/2/2008 22:58' prior: 0!
The submit type creates a button. When a submit button is pressed it automatically submits the form.!

WASubmitButtonTag subclass: #WACancelButtonTag
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Seaside-Canvas-Tags'!

!WACancelButtonTag methodsFor: 'callbacks' stamp: 'lr 1/4/2007 09:33'!
callback: aBlock
	self name: (canvas callbacks registerCancelActionCallback: aBlock)! !

WASubmitButtonTag subclass: #WAImageButtonTag
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Seaside-Canvas-Tags'!

!WAImageButtonTag commentStamp: 'pmm 7/28/2007 08:56' prior: 0!
Creates a graphical submit button. The value of the src attribute specifies the URI of the image that will decorate the button. For accessibility reasons, authors should provide alternate text for the image via the alt attribute. 

When a pointing device is used to click on the image, the form is submitted and the click coordinates passed to the server. The x value is measured in pixels from the left of the image, and the y value in pixels from the top of the image. The submitted data includes name.x=x-value and name.y=y-value where "name" is the value of the name attribute, and x-value and y-value are the x and y coordinate values, respectively.!

!WAImageButtonTag methodsFor: 'callbacks' stamp: 'lr 1/4/2007 09:33'!
callback: aBlock
	self name: (canvas callbacks registerImageCallback: aBlock)! !

!WAImageButtonTag methodsFor: 'attributes' stamp: 'lr 1/4/2007 09:33'!
extent: aPoint
	self width: aPoint x; height: aPoint y! !

!WAImageButtonTag methodsFor: 'attributes' stamp: 'lr 1/4/2007 09:33'!
form: aForm
	self extent: aForm extent.
	self url: (canvas context urlForDocument: aForm)! !

!WAImageButtonTag methodsFor: 'attributes' stamp: 'lr 8/3/2006 08:43'!
height: aNumber
	self attributes at: 'height' put: aNumber! !

!WAImageButtonTag methodsFor: 'attributes' stamp: 'lr 4/10/2007 09:55'!
resourceUrl: aString
	self url: (canvas context absoluteUrlForResource: aString)! !

!WAImageButtonTag methodsFor: 'accessing' stamp: 'lr 9/23/2005 13:49'!
type
	^ 'image'! !

!WAImageButtonTag methodsFor: 'attributes' stamp: 'lr 1/4/2007 09:33'!
url: aString
	self attributes at: 'src' put: aString! !

!WAImageButtonTag methodsFor: 'attributes' stamp: 'lr 8/3/2006 08:43'!
width: aNumber
	self attributes at: 'width' put: aNumber! !

!WASubmitButtonTag methodsFor: 'callbacks' stamp: 'lr 1/4/2007 09:32'!
callback: aBlock
	self name: (canvas callbacks registerActionCallback: aBlock)! !

!WASubmitButtonTag methodsFor: 'testing' stamp: 'lr 4/11/2008 08:48'!
canHaveShortcut
	^ true! !

!WASubmitButtonTag methodsFor: 'callbacks' stamp: 'lr 3/29/2008 16:41'!
on: aSymbol of: anObject
	self value: (self labelForSelector: aSymbol of: anObject).
	self callback: (MessageSend receiver: anObject selector: aSymbol)! !

!WASubmitButtonTag methodsFor: 'accessing' stamp: 'lr 9/23/2005 13:49'!
type
	^ 'submit'! !

!WASubmitButtonTag methodsFor: 'public' stamp: 'lr 8/30/2008 16:22'!
with: anObject 
	anObject isNil ifFalse: [ self value: anObject seasideString ].
	super with: nil! !

WATagBrush subclass: #WAGenericTag
	instanceVariableNames: 'tag'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Seaside-Canvas'!

!WAGenericTag commentStamp: 'pmm 6/1/2008 00:40' prior: 0!
A WAGenericTag is the class for all tags that do not have a class of their own.

Instance Variables
	tag:		<String>

tag
	- the name of the tag
!

WAGenericTag subclass: #WAEditTag
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Seaside-Canvas-Tags'!

!WAEditTag commentStamp: 'pmm 3/28/2007 10:40' prior: 0!
INS and DEL are used to markup sections of the document that have been inserted or deleted with respect to a different version of a document (e.g., in draft legislation where lawmakers need to view the changes).

These two elements are unusual for HTML in that they may serve as either block-level or inline elements (but not both). They may contain one or more words within a paragraph or contain one or more block-level elements such as paragraphs, lists and tables.

This example could be from a bill to change the legislation for how many deputies a County Sheriff can employ from 3 to 5.

<P>
  A Sheriff can employ <DEL>3</DEL><INS>5</INS> deputies.
</P>

The INS and DEL elements must not contain block-level content when these elements behave as inline elements.!

!WAEditTag methodsFor: 'attributes' stamp: 'lr 3/24/2008 13:01'!
cite: aUrl 
	"The value of this attribute is a URI that designates a source document or message. This attribute is intended to point to information explaining why a document was changed."
	self attributes 
		at: 'cite'
		put: aUrl! !

!WAEditTag methodsFor: 'attributes' stamp: 'lr 3/24/2008 13:01'!
datetime: anObject 
	"The value of this attribute specifies the date and time when the change was made.
	ISO date format"
	self attributes 
		at: 'datetime'
		put: anObject! !

!WAGenericTag class methodsFor: 'instance-creation' stamp: 'pmm 1/26/2007 23:05'!
tag: aString
	^ self new initializeWithTag: aString! !

!WAGenericTag methodsFor: 'initialize-release' stamp: 'avi 6/13/2004 00:40'!
initializeWithTag: aString
	tag := aString! !

!WAGenericTag methodsFor: 'accessing' stamp: 'avi 6/13/2004 00:40'!
tag
	^ tag! !

WATagBrush subclass: #WAHeadingTag
	instanceVariableNames: 'level'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Seaside-Canvas-Tags'!

!WAHeadingTag commentStamp: 'pmm 6/30/2007 00:44' prior: 0!
A heading element briefly describes the topic of the section it introduces. Heading information may be used by user agents, for example, to construct a table of contents for a document automatically.

There are six levels of headings in HTML with H1 as the most important and H6 as the least. Visual browsers usually render more important headings in larger fonts than less important ones.!

!WAHeadingTag methodsFor: 'initialization' stamp: 'lr 6/6/2007 14:02'!
initialize
	super initialize.
	self level1! !

!WAHeadingTag methodsFor: 'accessing' stamp: 'lr 4/9/2007 18:26'!
level
	^ level! !

!WAHeadingTag methodsFor: 'conveniance' stamp: 'lr 6/6/2007 14:02'!
level1
	level := 1! !

!WAHeadingTag methodsFor: 'conveniance' stamp: 'lr 6/6/2007 14:02'!
level2
	level := 2! !

!WAHeadingTag methodsFor: 'conveniance' stamp: 'lr 6/6/2007 14:02'!
level3
	level := 3! !

!WAHeadingTag methodsFor: 'conveniance' stamp: 'lr 6/6/2007 14:02'!
level4
	level := 4! !

!WAHeadingTag methodsFor: 'conveniance' stamp: 'lr 6/6/2007 14:02'!
level5
	level := 5! !

!WAHeadingTag methodsFor: 'conveniance' stamp: 'lr 6/6/2007 14:02'!
level6
	level := 6! !

!WAHeadingTag methodsFor: 'accessing' stamp: 'lr 6/6/2007 14:01'!
level: anInteger
	level := (anInteger max: 1) min: 6! !

!WAHeadingTag methodsFor: 'accessing' stamp: 'pmm 8/31/2008 14:33'!
tag
	^ 'h', self level seasideString! !

WATagBrush subclass: #WAHorizontalRuleTag
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Seaside-Canvas-Tags'!

!WAHorizontalRuleTag commentStamp: 'pmm 9/22/2007 22:24' prior: 0!
The HR element causes a horizontal rule to be rendered by visual user agents.

The amount of vertical space inserted between a rule and the content that surrounds it depends on the user agent.!

!WAHorizontalRuleTag methodsFor: 'testing' stamp: 'lr 3/26/2007 21:10'!
isClosed
	^ true! !

!WAHorizontalRuleTag methodsFor: 'accessing' stamp: 'lr 3/26/2007 21:08'!
tag
	^ 'hr'! !

WATagBrush subclass: #WAIframeTag
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Seaside-Canvas-Tags'!

!WAIframeTag commentStamp: 'pmm 9/22/2007 22:25' prior: 0!
The IFRAME element allows authors to insert a frame within a block of text. Inserting an inline frame within a section of text is much like inserting an object via the OBJECT element: they both allow you to insert an HTML document in the middle of another, they may both be aligned with surrounding text, etc. 

The information to be inserted inline is designated by the src attribute of this element. The contents of the IFRAME element, on the other hand, should only be displayed by user agents that do not support frames or are configured not to display frames.

Inline frames may not be resized.!

!WAIframeTag methodsFor: 'convenience' stamp: 'pmm 5/23/2007 19:05'!
contents: anObject
	self url: (canvas urlForAction: [ self performRendering: anObject ])! !

!WAIframeTag methodsFor: 'convenience' stamp: 'lr 8/9/2006 08:34'!
document: anObject
	self document: anObject mimeType: nil! !

!WAIframeTag methodsFor: 'convenience' stamp: 'lr 8/9/2006 08:31'!
document: anObject mimeType: aMimeString
	self document: anObject mimeType: aMimeString fileName: nil! !

!WAIframeTag methodsFor: 'convenience' stamp: 'lr 10/28/2007 14:42'!
document: anObject mimeType: aMimeString fileName: aFileNameString
	self url: (canvas context
		urlForDocument: anObject
		mimeType: aMimeString
		fileName: aFileNameString)! !

!WAIframeTag methodsFor: 'attributes' stamp: 'pmm 9/22/2007 22:26'!
height: anInteger
	"The height of the inline frame."
	self attributes at: 'height' put: anInteger! !

!WAIframeTag methodsFor: 'attributes' stamp: 'pmm 9/22/2007 22:26'!
name: aString
	"This attribute assigns a name to the current frame. This name may be used as the target of subsequent links."
	self attributes at: 'name' put: aString! !

!WAIframeTag methodsFor: 'private' stamp: 'pmm 11/2/2008 16:15'!
performRendering: anObject
	| innerContext docRoot document response renderer |
	innerContext := canvas context copy.
	docRoot := WAHtmlRoot context: innerContext.
	docRoot base target: '_top'.
	response := WAResponse new.
	response doNotCache.
	document := self requestContext entryPoint newDocumentOn: response stream.
	canvas context document: document.
	docRoot open: document.
	renderer := canvas species context: canvas context.
	renderer render: anObject;
		 flush.
	docRoot close: document.
	self requestContext respond: response! !

!WAIframeTag methodsFor: 'accessing' stamp: 'lr 3/8/2006 23:25'!
tag
	^ 'iframe'! !

!WAIframeTag methodsFor: 'attributes' stamp: 'lr 3/24/2008 13:01'!
url: aString 
	self attributes 
		at: 'src'
		put: aString! !

!WAIframeTag methodsFor: 'attributes' stamp: 'pmm 9/22/2007 22:26'!
width: anInteger
	"The width of the inline frame."
	self attributes at: 'width' put: anInteger! !

WATagBrush subclass: #WAImageTag
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Seaside-Canvas-Tags'!

!WAImageTag commentStamp: 'pmm 9/22/2007 22:28' prior: 0!
The IMG element embeds an image in the current document at the location of the element's definition. The IMG element has no content; it is usually replaced inline by the image designated by the src attribute, the exception being for left or right-aligned images that are "floated" out of line.

The alt attribute specifies alternate text that is rendered when the image cannot be displayed (see below for information on how to specify alternate text ). User agents must render alternate text when they cannot support images, they cannot support a certain image type or when they are configured not to display images.

Seaside per default sets the alternate text to an empty string. This helps validation of the page.!

!WAImageTag methodsFor: 'attributes' stamp: 'lr 10/28/2007 14:42'!
altText: aString
	"The alt attribute provides a short description of the image."
	self attributes at: 'alt' put: aString! !

!WAImageTag methodsFor: 'accessing' stamp: 'lr 6/20/2007 10:52'!
document: aDocument
	self document: aDocument mimeType: nil! !

!WAImageTag methodsFor: 'accessing' stamp: 'lr 6/20/2007 10:52'!
document: aDocument mimeType: mimeType
	self document: aDocument mimeType: mimeType fileName: nil! !

!WAImageTag methodsFor: 'accessing' stamp: 'lr 6/20/2007 10:52'!
document: aDocument mimeType: mimeType fileName: fileName
	self url: (canvas context urlForDocument: aDocument mimeType: mimeType fileName: fileName)! !

!WAImageTag methodsFor: 'accessing' stamp: 'lr 3/20/2007 14:26'!
form: aForm
	self document: aForm! !

!WAImageTag methodsFor: 'attributes' stamp: 'lr 10/28/2007 14:42'!
height: anInteger
	self attributes at: 'height' put: anInteger! !

!WAImageTag methodsFor: 'initialize-release' stamp: 'lr 6/20/2007 10:53'!
initialize
	super initialize.
	self altText: ''! !

!WAImageTag methodsFor: 'testing' stamp: 'lr 3/27/2007 11:06'!
isClosed
	^ true! !

!WAImageTag methodsFor: 'accessing' stamp: 'lr 4/10/2007 09:56'!
resourceUrl: aString
	self url: (canvas context absoluteUrlForResource: aString)! !

!WAImageTag methodsFor: 'initialize-release' stamp: 'lr 3/24/2008 13:01'!
setParent: aBrush canvas: aCanvas 
	super 
		setParent: aBrush
		canvas: aCanvas.
	(aBrush isKindOf: WAImageMapTag) ifTrue: [ 
		"in case the parent is an image map we need to set this attribute"
		self attributes 
			at: 'ismap'
			put: true ]! !

!WAImageTag methodsFor: 'deprecated' stamp: 'jf 9/24/2008 10:44'!
src: aString
	self seasideDeprecatedApi: 'WAImageTag>>src:'
		details: 'Use WAImageTag>>url:.'.
	self url: aString! !

!WAImageTag methodsFor: 'accessing' stamp: 'ac 3/16/2005 12:00'!
tag
	^ 'img'! !

!WAImageTag methodsFor: 'attributes' stamp: 'lr 6/20/2007 10:53'!
url: aString
	self attributes at: 'src' put: aString! !

!WAImageTag methodsFor: 'attributes' stamp: 'lr 10/28/2007 14:42'!
width: anInteger
	self attributes at: 'width' put: anInteger! !

WATagBrush subclass: #WAKeyGeneratorTag
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Seaside-Canvas-Tags'!

!WAKeyGeneratorTag commentStamp: 'pmm 11/11/2008 22:49' prior: 0!
A WAKeygenTag is allows the user agent to generate a secure key and submits the public key.

Supported by:
    * Netscape 3, 4, 6, 7
    * Mozilla 1
    * Firefox 1
    * Opera 3, 4, 5, 6, 7, 8
    * Safari 1 

=== Links for more info ===
http://lists.whatwg.org/pipermail/whatwg-whatwg.org/attachments/20080714/07ea5534/attachment.txt
http://wp.netscape.com/eng/security/comm4-keygen.html
http://wp.netscape.com/eng/security/ca-interface.html
http://webdesign.about.com/od/htmltags/p/bltags_keygen.htm
http://eskimonorth.com/~bloo/indexdot/html/tagpages/k/keygen.htm
http://web.archive.org/web/20040710102131/devedge.netscape.com/library/manuals/1998/htmlguide/
http://64.233.183.104/search?q=cache:TExfCslqOcwJ:www.html4newbies.com/Keygen.php
http://www.di.unito.it/~rabser/ssleay/x509_netscape.html
http://msdn.microsoft.com/en-us/library/cc214558.aspx - Something that may look like the MS version of the keygen attribute
!

!WAKeyGeneratorTag methodsFor: 'callbacks' stamp: 'pmm 11/11/2008 22:50'!
callback: aBlock
	self name: (canvas callbacks registerCallback: aBlock)! !

!WAKeyGeneratorTag methodsFor: 'attributes' stamp: 'pmm 11/11/2008 22:52'!
challenge: aString
	"Specifies the challenge string to be packaged with the public key for use in verification of the form submission. If no challenge string is provided, then it is encoded as an 'IA5STRING' of length zero. "
	self attributeAt: 'challenge' put: aString! !

!WAKeyGeneratorTag methodsFor: 'testing' stamp: 'pmm 11/11/2008 22:59'!
isClosed
	^true! !

!WAKeyGeneratorTag methodsFor: 'attributes' stamp: 'pmm 11/11/2008 22:53'!
keyType: aString
	"This attribute specifies the type of encryption security method to use to verify the receiver. 
    Values: 'RSA' (default) or 'DSA'"
	self attributeAt: 'keytype' put: aString! !

!WAKeyGeneratorTag methodsFor: 'attributes' stamp: 'pmm 11/11/2008 23:10'!
name: aString
	"The same as WAFormInputTag >> #name:"
	self attributeAt: 'name' put: aString! !

!WAKeyGeneratorTag methodsFor: 'attributes' stamp: 'pmm 11/11/2008 22:55'!
pqg: aString
	"This attribute appears to be a security string to be sent and verified."
	self attributeAt: 'pqg' put: aString! !

!WAKeyGeneratorTag methodsFor: 'accessing' stamp: 'pmm 11/11/2008 23:05'!
tag
	^ 'keygen'! !

WATagBrush subclass: #WALabelTag
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Seaside-Canvas-Tags'!

!WALabelTag commentStamp: 'obi 4/2/2008 22:38' prior: 0!
Defines a label to a control. If you click the text within the label element,  it is supposed to toggle the control.!

!WALabelTag methodsFor: 'attributes' stamp: 'lr 3/24/2008 13:01'!
for: aString 
	self attributes 
		at: 'for'
		put: aString! !

!WALabelTag methodsFor: 'accessing' stamp: 'ac 7/9/2005 04:36'!
tag
	^ 'label'! !

WATagBrush subclass: #WAObjectTag
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Seaside-Canvas-Tags'!

!WAObjectTag commentStamp: 'obi 4/2/2008 22:46' prior: 0!
Defines an embedded object. Use this element to add multimedia to your XHTML page.
This element allows you to specify the data and parameters for objects inserted into HTML documents, and the code that can be used to display/manipulate that data.!

!WAObjectTag methodsFor: 'attributes' stamp: 'lr 3/24/2008 13:01'!
archive: aUrl 
	self attributes 
		at: 'archive'
		put: aUrl! !

!WAObjectTag methodsFor: 'attributes' stamp: 'lr 3/24/2008 13:01'!
classId: aUrl 
	self attributes 
		at: 'classid'
		put: aUrl! !

!WAObjectTag methodsFor: 'attributes' stamp: 'lr 3/24/2008 13:01'!
codebase: aUrl 
	self attributes 
		at: 'codebase'
		put: aUrl! !

!WAObjectTag methodsFor: 'attributes' stamp: 'lr 3/24/2008 13:01'!
codetype: aString 
	self attributes 
		at: 'codetype'
		put: aString! !

!WAObjectTag methodsFor: 'attributes' stamp: 'lr 3/24/2008 13:01'!
declare: aBoolean 
	self attributes 
		at: 'declare'
		put: aBoolean! !

!WAObjectTag methodsFor: 'attributes' stamp: 'lr 3/24/2008 13:01'!
height: anInteger 
	self attributes 
		at: 'height'
		put: anInteger! !

!WAObjectTag methodsFor: 'attributes' stamp: 'lr 3/24/2008 13:01'!
standby: aString 
	self attributes 
		at: 'standby'
		put: aString! !

!WAObjectTag methodsFor: 'accessing' stamp: 'pmm 2/11/2007 19:24'!
tag
	^ 'object'! !

!WAObjectTag methodsFor: 'attributes' stamp: 'lr 3/24/2008 13:01'!
type: aString 
	self attributes 
		at: 'type'
		put: aString! !

!WAObjectTag methodsFor: 'attributes' stamp: 'lr 3/24/2008 13:01'!
url: aUrl 
	self attributes 
		at: 'data'
		put: aUrl! !

!WAObjectTag methodsFor: 'attributes' stamp: 'lr 3/24/2008 13:01'!
width: anInteger 
	self attributes 
		at: 'width'
		put: anInteger! !

WATagBrush subclass: #WAOptionGroupTag
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Seaside-Canvas-Tags'!

!WAOptionGroupTag commentStamp: 'pmm 4/24/2007 10:53' prior: 0!
The OPTGROUP element allows authors to group choices logically. This is particularly helpful when the user must choose from a long list of options; groups of related choices are easier to grasp and remember than a single long list of options.

It has crappy browser support and noone as ever used it. See WAInputTest >> #renderOptionGroupOn: for examples.!

!WAOptionGroupTag methodsFor: 'initialize-release' stamp: 'lr 2/16/2008 18:14'!
initialize
	super initialize.
	self label: ''! !

!WAOptionGroupTag methodsFor: 'attributes' stamp: 'lr 10/28/2007 14:42'!
label: aString
	"This attribute specifies the label for the option group."

	self attributes at: 'label' put: aString! !

!WAOptionGroupTag methodsFor: 'accessing' stamp: 'pmm 4/24/2007 10:49'!
tag
	^ 'optgroup'! !

WATagBrush subclass: #WAOptionTag
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Seaside-Canvas-Tags'!

!WAOptionTag commentStamp: 'obi 4/2/2008 22:45' prior: 0!
The option element defines an option in a drop-down list. !

!WAOptionTag methodsFor: 'callbacks' stamp: 'lr 6/6/2007 19:42'!
callback: aBlock
	self value: (canvas callbacks registerCallback: aBlock)! !

!WAOptionTag methodsFor: 'attributes' stamp: 'lr 10/28/2007 14:42'!
label: aString
	"This attribute allows authors to specify a shorter label for an option than the content of the OPTION element. When specified, user agents should use the value of this attribute rather than the content of the OPTION element as the option label."

	self attributes at: 'label' put: aString! !

!WAOptionTag methodsFor: 'attributes' stamp: 'lr 6/6/2007 19:42'!
selected: aBoolean
	self attributes at: 'selected' put: aBoolean! !

!WAOptionTag methodsFor: 'accessing' stamp: 'ac 3/17/2005 17:13'!
tag
	^ 'option'! !

!WAOptionTag methodsFor: 'attributes' stamp: 'lr 6/6/2007 19:42'!
value: aString
	self attributes at: 'value' put: aString! !

WATagBrush subclass: #WAParameterTag
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Seaside-Canvas-Tags'!

!WAParameterTag commentStamp: 'obi 4/2/2008 22:45' prior: 0!
The param element allows you to specify the run-time settings for an object inserted into XHTML documents.!

!WAParameterTag methodsFor: 'attributes' stamp: 'lr 3/24/2008 13:01'!
name: aString 
	self attributes 
		at: 'name'
		put: aString! !

!WAParameterTag methodsFor: 'accessing' stamp: 'pmm 2/11/2007 20:12'!
tag
	^ 'param'! !

!WAParameterTag methodsFor: 'attributes' stamp: 'lr 3/24/2008 13:01'!
type: aString 
	self attributes 
		at: 'type'
		put: aString! !

!WAParameterTag methodsFor: 'attributes' stamp: 'lr 3/24/2008 13:01'!
value: aString 
	self attributes 
		at: 'value'
		put: aString! !

WATagBrush subclass: #WARubyTextTag
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Seaside-Canvas-Tags'!

!WARubyTextTag commentStamp: 'obi 4/2/2008 22:50' prior: 0!
This element works with the <ruby> element. 
The text following <rt> appears as a note to the base text displayed by the 
<ruby> element.!

!WARubyTextTag methodsFor: 'attributes' stamp: 'lr 3/24/2008 13:01'!
span: anInteger 
	self attributes 
		at: 'rbspan'
		put: anInteger! !

!WARubyTextTag methodsFor: 'accessing' stamp: 'pmm 9/18/2006 19:00'!
tag
	^ 'rt'! !

WATagBrush subclass: #WAScriptTag
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Seaside-Canvas-Tags'!

!WAScriptTag commentStamp: 'obi 4/2/2008 22:51' prior: 0!
Defines a script, such as a JavaScript.!

!WAScriptTag methodsFor: 'private' stamp: 'lr 3/27/2007 11:57'!
after
	self document nextPutAll: '/*]]>*/'! !

!WAScriptTag methodsFor: 'private' stamp: 'lr 3/27/2007 11:56'!
before
	self document nextPutAll: '/*<!![CDATA[*/'! !

!WAScriptTag methodsFor: 'attributes' stamp: 'lr 3/24/2008 13:01'!
defer
	"When set, this boolean attribute provides a hint to the user agent that the script is not going to generate any document content (e.g., no 'document.write' in javascript) and thus, the user agent can continue parsing and rendering."
	self attributes 
		at: 'defer'
		put: true! !

!WAScriptTag methodsFor: 'attributes' stamp: 'lr 4/10/2007 09:55'!
resourceUrl: aString
	self url: (canvas context absoluteUrlForResource: aString)! !

!WAScriptTag methodsFor: 'accessing' stamp: 'dwj 6/10/2005 13:10'!
tag
	^ 'script'! !

!WAScriptTag methodsFor: 'attributes' stamp: 'pmm 8/26/2006 00:05'!
url: aString
	self attributes at: 'src' put: aString! !

!WAScriptTag methodsFor: 'public' stamp: 'pmm 8/31/2008 14:48'!
with: aString 
	self attributes 
		at: 'type'
		ifAbsentPut: [ 'text/javascript' ].
	super with: [
		aString isNil
			ifFalse: [ self document nextPutAll: aString seasideString ] ]! !

WATagBrush subclass: #WATableCellTag
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Seaside-Canvas-Tags'!

!WATableCellTag methodsFor: 'attributes' stamp: 'lr 3/24/2008 13:44'!
align: aString
	"This attribute specifies the alignment of data and the justification of text in a cell. Possible values:
	
- left: Left-flush data/Left-justify text. This is the default value for table data.
- center: Center data/Center-justify text. This is the default value for table headers.
- right: Right-flush data/Right-justify text.
- justify: Double-justify text.
- char: Align text around a specific character. If a user agent doesn't support character alignment, behavior in - the presence of this value is unspecified."

	self attributes at: 'align' put: aString! !

!WATableCellTag methodsFor: 'attributes' stamp: 'lr 3/24/2008 13:45'!
character: aString
	"This attribute specifies a single character within a text fragment to act as an axis for alignment. The default value for this attribute is the decimal point character for the current language as set by the lang attribute. User agents are not required to support this attribute."
	
	self attributes at: 'char' put: aString! !

!WATableCellTag methodsFor: 'attributes' stamp: 'lr 3/24/2008 13:45'!
characterOffset: anInteger
	"When present, this attribute specifies the offset to the first occurrence of the alignment character on each line. If a line doesn't include the alignment character, it should be horizontally shifted to end at the alignment position."

	self attributes at: 'charoff' put: anInteger! !

!WATableCellTag methodsFor: 'attributes' stamp: 'lr 3/24/2008 13:44'!
verticalAlign: aString
	"This attribute specifies the vertical position of data within a cell. Possible values:

- top: Cell data is flush with the top of the cell.
- middle: Cell data is centered vertically within the cell. This is the default value.
- bottom: Cell data is flush with the bottom of the cell.
- baseline: All cells in the same row as a cell whose valign attribute has this value should have their textual data positioned so that the first text line occurs on a baseline common to all cells in the row. This constraint does not apply to subsequent text lines in these cells."
		
	self attributes at: 'valign' put: aString! !

WATableCellTag subclass: #WATableColumnGroupTag
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Seaside-Canvas-Tags'!

!WATableColumnGroupTag commentStamp: 'lr 3/24/2008 13:19' prior: 0!
WATableColumnGroupTag groups a set of col elements. It allows you to group several semantically related columns together.!

!WATableColumnGroupTag methodsFor: 'attributes' stamp: 'lr 3/24/2008 13:37'!
span: anInteger
	"This attribute must be an integer > 0; the default value is 1. This specifies the number of columns in a colgroup, or specifies the number of columns spanned by the col element."
	
	self attributes at: 'span' put: anInteger! !

!WATableColumnGroupTag methodsFor: 'accessing' stamp: 'mb 12/8/2006 15:51'!
tag
	^ 'colgroup'! !

!WATableColumnGroupTag methodsFor: 'attributes' stamp: 'lr 3/24/2008 13:38'!
width: anInteger
	"The width attribute specifies the width of the columns"
	
	self attributes at: 'width' put: anInteger! !

WATableColumnGroupTag subclass: #WATableColumnTag
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Seaside-Canvas-Tags'!

!WATableColumnTag commentStamp: 'lr 3/24/2008 13:48' prior: 0!
WATableColumnTag elements define the alignment properties for cells in one or more columns.!

!WATableColumnTag methodsFor: 'testing' stamp: 'lr 3/27/2007 11:06'!
isClosed
	^ true! !

!WATableColumnTag methodsFor: 'accessing' stamp: 'mb 12/8/2006 15:51'!
tag
	^ 'col'! !

WATableCellTag subclass: #WATableDataTag
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Seaside-Canvas-Tags'!

!WATableDataTag commentStamp: 'obi 4/2/2008 22:41' prior: 0!
The WATableDataTag defines a cell in a table.

!

!WATableDataTag methodsFor: 'attributes' stamp: 'lr 3/24/2008 17:01'!
abbreviation: aString
	"This attribute should be used to provide an abbreviated form of the cell's content, and may be rendered by user agents when appropriate in place of the cell's content. Abbreviated names should be short since user agents may render them repeatedly. For instance, speech synthesizers may render the abbreviated headers relating to a particular cell before rendering that cell's content."
	
	self attributes at: 'abbr' put: aString! !

!WATableDataTag methodsFor: 'attributes' stamp: 'lr 3/24/2008 17:01'!
axis: aString
	"This attribute may be used to place a cell into conceptual categories that can be considered to form axes in an n-dimensional space. User agents may give users access to these categories (e.g., the user may query the user agent for all cells that belong to certain categories, the user agent may present a table in the form of a table of contents, etc.). Please consult the section on categorizing cells for more information. The value of this attribute is a comma-separated list of category names."
	
	self attributes at: 'axis' put: aString! !

!WATableDataTag methodsFor: 'attributes' stamp: 'lr 3/24/2008 13:35'!
colSpan: anInteger
	"This attribute specifies the number of columns spanned by the current cell. The default value of this attribute is one. The value zero means that the cell spans all columns from the current column to the last column of the column group (colgroup) in which the cell is defined."
	
	self attributes at: 'colspan' put: anInteger! !

!WATableDataTag methodsFor: 'attributes' stamp: 'lr 3/24/2008 13:36'!
headers: aString
	"This attribute specifies the list of header cells that provide header information for the current data cell. The value of this attribute is a space-separated list of cell names; those cells must be named by setting their id attribute. Authors generally use the headers attribute to help non-visual user agents render header information about data cells (e.g., header information is spoken prior to the cell data), but the attribute may also be used in conjunction with style sheets. See also the scope attribute."
	
	self attributes at: 'headers' put: aString! !

!WATableDataTag methodsFor: 'attributes' stamp: 'lr 3/24/2008 13:35'!
rowSpan: anInteger
	"This attribute specifies the number of rows spanned by the current cell. The default value of this attribute is one. The value zero means that the cell spans all rows from the current row to the last row of the current table section (rowgroup) in which the cell is defined. thead, tbody, and tfoot elements are rowgroups."
	
	self attributes at: 'rowspan' put: anInteger! !

!WATableDataTag methodsFor: 'attributes' stamp: 'lr 3/24/2008 13:35'!
scope: aString
	"This attribute specifies the set of data cells for which the current header cell provides header information. This attribute may be used in place of the headers attribute, particularly for simple tables. When specified, this attribute must have one of the following values:

- row: The current cell provides header information for the rest of the row that contains it (see also the section on table directionality).
- col: The current cell provides header information for the rest of the column that contains it.
- rowgroup: The header cell provides header information for the rest of the row group that contains it.
- colgroup: The header cell provides header information for the rest of the column group that contains it."

	self attributes at: 'scope' put: aString! !

!WATableDataTag methodsFor: 'accessing' stamp: 'ac 3/17/2005 22:09'!
tag
	^ 'td'! !

WATableDataTag subclass: #WATableHeadingTag
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Seaside-Canvas-Tags'!

!WATableHeadingTag commentStamp: 'obi 4/2/2008 22:42' prior: 0!
The WATableHeadingTag defines a table header cell in a table. The text within the th element usually renders in bold.!

!WATableHeadingTag methodsFor: 'accessing' stamp: 'ac 3/17/2005 22:10'!
tag
	^ 'th'! !

WATagBrush subclass: #WATableTag
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Seaside-Canvas-Tags'!

!WATableTag commentStamp: 'obi 4/2/2008 22:42' prior: 0!
The <table> tag defines a table. Inside a <table> tag you can put table headers, table rows, table cells, and other tables.!

!WATableTag methodsFor: 'deprecated' stamp: 'jf 9/24/2008 10:44'!
align: aString
	self seasideDeprecatedApi: 'WATableTag>>align:'.
	self attributes at: 'align' put: aString! !

!WATableTag methodsFor: 'deprecated' stamp: 'jf 9/24/2008 10:43'!
border: anInteger
	self seasideDeprecatedApi: 'WATableTag>>border:'.
	self attributes at: 'border' put: anInteger! !

!WATableTag methodsFor: 'attributes' stamp: 'lr 12/30/2006 00:11'!
cellpadding: anInteger
	self attributes at: 'cellpadding' put: anInteger! !

!WATableTag methodsFor: 'attributes' stamp: 'lr 12/30/2006 00:11'!
cellspacing: anInteger
	self attributes at: 'cellspacing' put: anInteger! !

!WATableTag methodsFor: 'attributes' stamp: 'pmm 4/8/2007 00:07'!
frame: aString
	"frame = void|above|below|hsides|lhs|rhs|vsides|box|border [CI]
    This attribute specifies which sides of the frame surrounding a table will be visible. Possible values:

        * void: No sides. This is the default value.
        * above: The top side only.
        * below: The bottom side only.
        * hsides: The top and bottom sides only.
        * vsides: The right and left sides only.
        * lhs: The left-hand side only.
        * rhs: The right-hand side only.
        * box: All four sides.
        * border: All four sides."

	self attributes at: 'frame' put: aString! !

!WATableTag methodsFor: 'attributes' stamp: 'pmm 4/8/2007 00:06'!
rules: aString
	"rules = none|groups|rows|cols|all [CI]
    This attribute specifies which rules will appear between cells within a table. The rendering of rules is user agent dependent. Possible values:

        * none: No rules. This is the default value.
        * groups: Rules will appear between row groups (see THEAD, TFOOT, and TBODY) and column groups (see COLGROUP and COL) only.
        * rows: Rules will appear between rows only.
        * cols: Rules will appear between columns only.
        * all: Rules will appear between all rows and columns."
	self attributes at: 'rules' put: aString! !

!WATableTag methodsFor: 'attributes' stamp: 'mb 12/8/2006 15:03'!
summary: aString
	self attributes at: 'summary' put: aString! !

!WATableTag methodsFor: 'accessing' stamp: 'ac 3/16/2005 15:22'!
tag
	^ 'table'! !

!WATagBrush class methodsFor: 'code generation' stamp: 'pmm 9/12/2006 19:28'!
tag
	"WASelectTag tag"
	"WAGenericTag tag"
	^(self selectors includes: #tag)
		ifTrue: [ self new tag ]
		ifFalse: [ nil ]! !

!WATagBrush methodsFor: 'attributes-keyboard' stamp: 'lr 10/28/2007 14:42'!
accessKey: aString
	"Set a keyboard shortcut to access an element. An access key is a single character from the document character set.

	Pressing an access key assigned to an element gives focus to the element. The action that occurs when an element receives focus depends on the element. For example, when a user activates a link defined by the A element, the user agent generally follows the link. When a user activates a radio button, the user agent changes the value of the radio button. When the user activates a text field, it allows input, etc.

The following elements support the accesskey attribute: A, AREA, BUTTON, INPUT, LABEL, and LEGEND, and TEXTAREA.

	The invocation of access keys depends on the underlying system. For instance, on machines running MS Windows, one generally has to press the 'alt' key in addition to the access key. On Apple systems, one generally has to press the 'cmd' key in addition to the access key."

	self attributes at: 'accesskey' put: aString! !

!WATagBrush methodsFor: 'convenience' stamp: 'lr 4/9/2008 10:51'!
addShortcut: aString 
	self canHaveShortcut ifFalse: 
		[ self error: 'May not assign shortcut on items that are not clickable' ].
	self ensureId.
	self requestContext addLoadScript: (String streamContents: 
			[ :stream | 
			stream
				nextPutAll: 'addShortcut(''';
				nextPutAll: aString;
				nextPutAll: ''', ''';
				nextPutAll: self id;
				nextPutAll: ''')' ])! !

!WATagBrush methodsFor: 'private' stamp: 'lr 4/9/2007 13:18'!
after
	"This template method is called directly after rendering the content of the receiver."! !

!WATagBrush methodsFor: 'accessing-attributes' stamp: 'lr 3/23/2007 20:57'!
attributeAt: aKey
	^ self attributes at: aKey! !

!WATagBrush methodsFor: 'accessing-attributes' stamp: 'lr 3/23/2007 20:58'!
attributeAt: aKey ifAbsent: aBlock
	^ self attributes at: aKey ifAbsent: aBlock! !

!WATagBrush methodsFor: 'accessing-attributes' stamp: 'lr 3/23/2007 20:58'!
attributeAt: aKey ifAbsentPut: aBlock
	^ self attributes at: aKey ifAbsentPut: aBlock! !

!WATagBrush methodsFor: 'accessing-attributes' stamp: 'lr 3/23/2007 20:58'!
attributeAt: aKey ifPresent: aBlock
	^ self attributes at: aKey ifPresent: aBlock! !

!WATagBrush methodsFor: 'accessing-attributes' stamp: 'lr 3/23/2007 20:57'!
attributeAt: aKey put: aValue
	^ self attributes at: aKey put: aValue! !

!WATagBrush methodsFor: 'accessing' stamp: 'lr 3/23/2007 20:08'!
attributes
	^ attributes ifNil: [ attributes := WAHtmlAttributes new ]! !

!WATagBrush methodsFor: 'accessing' stamp: 'lr 10/28/2007 14:42'!
attributes: anHtmlAttributes
	attributes := anHtmlAttributes! !

!WATagBrush methodsFor: 'private' stamp: 'lr 4/9/2007 13:18'!
before
	"This template method is called directly before rendering the content of the receiver."! !

!WATagBrush methodsFor: 'testing' stamp: 'lr 4/11/2008 08:48'!
canHaveShortcut
	^ false! !

!WATagBrush methodsFor: 'attributes-core' stamp: 'lr 10/28/2007 14:42'!
class: aString
	"This attribute assigns one or more class names to an element; the element may be said to belong to these classes. A class name may be shared by several element instances. Multiple classes might be added to one brush."

	self attributes addClass: aString! !

!WATagBrush methodsFor: 'convenience' stamp: 'lr 10/28/2007 14:42'!
class: aString if: aBoolean
	"Adds the class aString aString to this element if aBoolean is true,

	Example

	html div
		class: 'important' if: self isImportant;
		with: self message"

	aBoolean
		ifTrue: [self class: aString]! !

!WATagBrush methodsFor: 'private' stamp: 'lr 10/28/2007 14:42'!
closeTag
	"Close the receiving tag onto the document."

	self isClosed ifFalse: [ self document closeTag: self tag ]! !

!WATagBrush methodsFor: 'convenience' stamp: 'avi 3/9/2006 17:41'!
confirm: aString
	self onClick: 'return confirm("', aString, '")'! !

!WATagBrush methodsFor: 'attributes-language' stamp: 'lr 10/28/2007 14:42'!
direction: aString
	"Set the text direction ltr (left-to-right) or right-to-left (rtl).

	Not supported on:
	APPLET, BASE, BASEFONT, BR, FRAME, FRAMESET, IFRAME, PARAM, SCRIPT"

	self attributes at: 'dir' put: aString! !

!WATagBrush methodsFor: 'convenience' stamp: 'lr 10/28/2007 14:42'!
disableEnter
	"pressing the 'enter' key in the form input element does not submit the form"
	self onEnter: ''! !

!WATagBrush methodsFor: 'attributes' stamp: 'pmm 5/17/2008 23:21'!
disabled
	"When set for a form control, this boolean attribute disables the control for user input.
When set, the disabled attribute has the following effects on an element:
	-	Disabled controls do not receive focus.
	-	Disabled controls are skipped in tabbing navigation.
	-	Disabled controls cannot be successful (= will not be submitted).
		
	The following elements support the disabled attribute: BUTTON, INPUT, OPTGROUP, OPTION, SELECT, and TEXTAREA. This one other example of functionality implemented way too high in the class hierarchy because we don't have traits."
	self disabled: true! !

!WATagBrush methodsFor: 'attributes' stamp: 'lr 6/6/2007 16:20'!
disabled: aBoolean
	self attributes at: 'disabled' put: aBoolean! !

!WATagBrush methodsFor: 'private' stamp: 'avi 6/13/2004 00:39'!
document
	^ canvas document! !

!WATagBrush methodsFor: 'public' stamp: 'lr 4/9/2007 16:31'!
ensureId
	"Answer the id of the receiving attribute. In case the receiver doesn't have an id yet, generate a new one."

	^ self attributes at: 'id' ifAbsentPut: [ canvas nextId ]! !

!WATagBrush methodsFor: 'accessing' stamp: 'pmm 11/21/2007 14:24'!
id
	^ self attributes at: 'id'! !

!WATagBrush methodsFor: 'attributes-core' stamp: 'lr 10/28/2007 14:42'!
id: aString
	"The id attribute assigns a identifier to an element. The id of an element must be unique within a document."

	self attributes at: 'id' put: aString! !

!WATagBrush methodsFor: 'testing' stamp: 'lr 10/28/2007 14:42'!
isClosed
	"Answer true if this tag should be closed immediately, such as <br />."

	^ false! !

!WATagBrush methodsFor: 'attributes' stamp: 'pmm 5/17/2008 23:19'!
isDisabled
	"answers whether the receiver is disabled"
	^self attributes
		at: 'disabled'
		ifAbsent: [ false ]! !

!WATagBrush methodsFor: 'attributes-language' stamp: 'lr 10/28/2007 14:42'!
language: aString
	"Set the language code."

	self attributes
		at: 'lang' put: aString;
		at: 'xml:lang' put: aString! !

!WATagBrush methodsFor: 'attributes-events' stamp: 'lr 4/9/2007 14:08'!
onBlur: aString
	"The onblurs event occurs when the element that is in focus, loses that focus."

	self attributes at: 'onblur' append: aString separator: ';'! !

!WATagBrush methodsFor: 'attributes-events' stamp: 'lr 4/9/2007 14:09'!
onChange: aString
	"The onchange event occurs when a select input element has a selection made or when a text input element has a change in the text."

	self attributes at: 'onchange' append: aString separator: ';'! !

!WATagBrush methodsFor: 'attributes-events' stamp: 'lr 6/7/2006 13:48'!
onClick: aString
	"The onclick event occurs when the pointing device button is clicked over an element."

	self attributes at: 'onclick' append: aString separator: ';'! !

!WATagBrush methodsFor: 'attributes-events' stamp: 'lr 6/7/2006 13:48'!
onDoubleClick: aString
	"The ondblclick event occurs when the pointing device button is double clicked over an element."

	self attributes at: 'ondblclick' append: aString separator: ';'! !

!WATagBrush methodsFor: 'convenience' stamp: 'lr 8/30/2008 16:22'!
onEnter: aString 
	"If 'enter' is pressed"
	self onKeyPress: 'if((window.event ? window.event.keyCode : event.which) == 13){' , aString seasideString , '; return false}; return true'! !

!WATagBrush methodsFor: 'convenience' stamp: 'lr 6/7/2006 13:48'!
onError: aString
	self attributes at: 'onerror' append: aString separator: ';'! !

!WATagBrush methodsFor: 'attributes-events' stamp: 'lr 6/7/2006 13:48'!
onFocus: aString
	"The onfocus event occurs when an element receives focus either by the pointing device or by tabbing navigation."

	self attributes at: 'onfocus' append: aString separator: ';'! !

!WATagBrush methodsFor: 'attributes-events' stamp: 'lr 6/7/2006 13:48'!
onKeyDown: aString
	"The onkeydown event occurs when a key is pressed down over an element."

	self attributes at: 'onkeydown' append: aString separator: ';'! !

!WATagBrush methodsFor: 'attributes-events' stamp: 'lr 6/7/2006 13:48'!
onKeyPress: aString
	"The onkeypress event occurs when a key is pressed and released over an element."

	self attributes at: 'onkeypress' append: aString separator: ';'! !

!WATagBrush methodsFor: 'attributes-events' stamp: 'lr 6/7/2006 13:48'!
onKeyUp: aString
	"The onkeyup event occurs when a key is released over an element."

	self attributes at: 'onkeyup' append: aString separator: ';'! !

!WATagBrush methodsFor: 'attributes-events' stamp: 'mb 6/21/2006 18:28'!
onLoad: aString
	"The onload event occurs when the user agent finishes loading a window."

	self attributes at: 'onload' append: aString separator: ';'! !

!WATagBrush methodsFor: 'attributes-events' stamp: 'lr 6/7/2006 13:48'!
onMouseDown: aString
	"The onmousedown event occurs when the pointing device button is pressed over an element."

	self attributes at: 'onmousedown' append: aString separator: ';'! !

!WATagBrush methodsFor: 'attributes-events' stamp: 'lr 6/7/2006 13:48'!
onMouseMove: aString
	"The onmousemove event occurs when the pointing device is moved while it is over an element."

	self attributes at: 'onmousemove' append: aString separator: ';'! !

!WATagBrush methodsFor: 'attributes-events' stamp: 'lr 6/7/2006 13:48'!
onMouseOut: aString
	"The onmouseout event occurs when the pointing device is moved away from an element."

	self attributes at: 'onmouseout' append: aString separator: ';'! !

!WATagBrush methodsFor: 'attributes-events' stamp: 'lr 6/7/2006 13:48'!
onMouseOver: aString
	"The onmouseover event occurs when the pointing device is moved onto an element."

	self attributes at: 'onmouseover' append: aString separator: ';'! !

!WATagBrush methodsFor: 'attributes-events' stamp: 'lr 6/7/2006 13:48'!
onMouseUp: aString
	"The onmouseup event occurs when the pointing device button is released over an element."

	self attributes at: 'onmouseup' append: aString separator: ';'! !

!WATagBrush methodsFor: 'attributes-events' stamp: 'lr 6/7/2006 13:49'!
onReset: aString
	"The onreset event occurs when a form is reset."

	self attributes at: 'onreset' append: aString separator: ';'! !

!WATagBrush methodsFor: 'convenience' stamp: 'lr 6/7/2006 13:49'!
onResize: aString
	self attributes at: 'onresize' append: aString separator: ';'! !

!WATagBrush methodsFor: 'convenience' stamp: 'lr 6/7/2006 13:49'!
onScroll: aString
	self attributes at: 'onscroll' append: aString separator: ';'! !

!WATagBrush methodsFor: 'attributes-events' stamp: 'lr 6/7/2006 13:49'!
onSelect: aString
	"The onselect event occurs when a user selects some text in a text field."

	self attributes at: 'onselect' append: aString separator: ';'! !

!WATagBrush methodsFor: 'attributes-events' stamp: 'lr 6/7/2006 13:49'!
onSubmit: aString
	"The onsubmit event occurs when a form is submitted."

	self attributes at: 'onsubmit' append: aString separator: ';'! !

!WATagBrush methodsFor: 'attributes-events' stamp: 'lr 6/7/2006 13:49'!
onUnload: aString
	"The onunload event occurs when the user agent removes a document from a window."

	self attributes at: 'onunload' append: aString separator: ';'! !

!WATagBrush methodsFor: 'private' stamp: 'lr 10/28/2007 14:42'!
openTag
	"Open the receiving and all associated attributes onto the document."

	self document openTag: self tag attributes: attributes closed: self isClosed! !

!WATagBrush methodsFor: 'attributes-core' stamp: 'lr 4/9/2007 16:13'!
style: aString
	"This attribute offers optional CSS style information. The attribute is deprecated and should be avoided in favor of an external stylesheet. Multiple styles might be added to one brush."

	self attributes addStyle: aString! !

!WATagBrush methodsFor: 'attributes-keyboard' stamp: 'lr 10/28/2007 14:42'!
tabIndex: aString
	"This attribute specifies the position of the current element in the tabbing order for the current document. This value must be a number between 0 and 32767.

	The tabbing order defines the order in which elements will receive focus when navigated by the user via the keyboard. The tabbing order may include elements nested within other elements.

	The following elements support the tabindex attribute: A, AREA, BUTTON, INPUT, OBJECT, SELECT, and TEXTAREA."

	self attributes at: 'tabindex' put: aString! !

!WATagBrush methodsFor: 'accessing' stamp: 'avi 6/13/2004 00:30'!
tag
	self subclassResponsibility! !

!WATagBrush methodsFor: 'attributes-core' stamp: 'lr 10/28/2007 14:42'!
title: aString
	"This attribute offers advisory information about the element for which it is set. Visual browsers frequently display the title as a 'tool tip'."

	self attributes at: 'title' put: aString! !

!WATagBrush methodsFor: 'public' stamp: 'jf 11/4/2008 12:40'!
with: anObject
	"Render anObject into the receiver. Make sure that you call #with: last in the cascade, as this method will serialize the tag onto the output document."

	self openTag.
	super with: [
		self before.
		canvas render: anObject.
		self after ].
	self closeTag! !

!WATagBrush methodsFor: 'convenience' stamp: 'pmm 5/10/2007 10:16'!
withLineBreaks: aString
	"Renders text preserving line breaks."
	self with: [canvas withLineBreaks: aString]! !

WAObject subclass: #WARadioGroup
	instanceVariableNames: 'canvas key'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Seaside-Canvas'!

!WARadioGroup commentStamp: 'pmm 6/1/2008 00:47' prior: 0!
A WARadioGroup is a container for several related radio buttons. It must be used to create radio buttons.

Example:
| group |
group := html radioGroup.
group radioButton
	selected: aBoolean;
	callback: [ self someThing ].
!

!WARadioGroup class methodsFor: 'accessing' stamp: 'ac 7/6/2005 11:30'!
canvas: aRenderCanvas
	^ self basicNew initializeWithCanvas: aRenderCanvas! !

!WARadioGroup methodsFor: 'initialize-release' stamp: 'lr 10/28/2007 14:42'!
initializeWithCanvas: aRenderCanvas
	canvas := aRenderCanvas.
	key := aRenderCanvas callbacks registerDispatchCallback! !

!WARadioGroup methodsFor: 'accessing' stamp: 'ac 7/6/2005 11:32'!
key
	^ key! !

!WARadioGroup methodsFor: 'public api' stamp: 'ac 7/6/2005 11:31'!
radioButton
	^ canvas radioButton group: self! !

WARenderer subclass: #WACanvas
	instanceVariableNames: 'currentBrush parentBrush'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Seaside-Canvas'!

!WACanvas commentStamp: 'pmm 4/15/2007 22:05' prior: 0!
This is the superclass of all canvas. It's a rendering interfact that generates brushes (see WABrush).

Subclass this class, if you want to generate an XML dialect.!

!WACanvas methodsFor: 'public' stamp: 'avi 6/17/2004 12:36'!
brush: aBrush
	self flush.
	currentBrush := aBrush.
	aBrush setParent: parentBrush canvas: self.
	^ aBrush! !

!WACanvas methodsFor: 'private' stamp: 'jf 10/19/2008 02:09'!
flush
	super flush.
	currentBrush ifNotNil:
		[currentBrush close.
		currentBrush := nil]! !

!WACanvas methodsFor: 'private' stamp: 'lr 6/6/2007 10:32'!
insert: aBlock
	| oldBrush |
	oldBrush := currentBrush.
	currentBrush := nil.
	aBlock value.
	self flush.
	currentBrush := oldBrush! !

!WACanvas methodsFor: 'private' stamp: 'jf 11/4/2008 12:41'!
nest: aBlock
	parentBrush := currentBrush.
	currentBrush := nil.
	self render: aBlock.
	self flush.
	parentBrush := parentBrush parent! !

!WACanvas methodsFor: 'public' stamp: 'jf 10/19/2008 01:57'!
render: anObject
	self flush.
	super render: anObject! !

!WACanvas methodsFor: 'public' stamp: 'jf 10/19/2008 01:56'!
text: anObject
	self flush.
	super text: anObject! !

WACanvas subclass: #WATagCanvas
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Seaside-Canvas'!

WATagCanvas subclass: #WAHtmlCanvas
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Seaside-Canvas'!

!WAHtmlCanvas commentStamp: 'jf 7/16/2008 12:16' prior: 0!
This is an abstract canvas that knows about HTML but nothing about callbacks. Subclasses must implement #document.!

!WAHtmlCanvas methodsFor: 'tags-block' stamp: 'lr 4/9/2007 17:01'!
abbreviated
	"Defines an abbreviation, such as 'M.', 'Inc.', 'et al.', 'etc.'"

	^ self tag: 'abbr'! !

!WAHtmlCanvas methodsFor: 'tags-block' stamp: 'pmm 9/30/2006 20:06'!
abbreviated: aBlock
	self abbreviated with: aBlock! !

!WAHtmlCanvas methodsFor: 'tags-block' stamp: 'lr 10/28/2007 14:42'!
acronym
	"Defines an acronym, such as 'GmbH', 'NATO', and 'F.B.I.'"

	^ self tag: 'acronym'! !

!WAHtmlCanvas methodsFor: 'tags-block' stamp: 'mb 3/24/2006 13:42'!
acronym: aBlock
	self acronym with: aBlock! !

!WAHtmlCanvas methodsFor: 'tags-block' stamp: 'lr 10/28/2007 14:42'!
address
	"Defines an address element."

	^ self tag: 'address'! !

!WAHtmlCanvas methodsFor: 'tags-block' stamp: 'pmm 12/24/2005 17:44'!
address: aBlock
	self address with: aBlock! !

!WAHtmlCanvas methodsFor: 'tags-input' stamp: 'lr 10/28/2007 14:42'!
anchor
	"Defines an anchor."

	^ self brush: WAAnchorTag new! !

!WAHtmlCanvas methodsFor: 'tags-input' stamp: 'lr 4/9/2007 17:23'!
anchor: aBlock
	self anchor with: aBlock! !

!WAHtmlCanvas methodsFor: 'tags-format' stamp: 'lr 10/28/2007 14:42'!
big
	"Defines big text."

	^ self tag: 'big'! !

!WAHtmlCanvas methodsFor: 'tags-format' stamp: 'ac 2/21/2006 14:36'!
big: aBlock
	self big with: aBlock! !

!WAHtmlCanvas methodsFor: 'tags-block' stamp: 'lr 4/9/2007 17:02'!
blockquote
	"Defines a long quotation."

	^ self tag: 'blockquote'! !

!WAHtmlCanvas methodsFor: 'tags-block' stamp: 'avi 6/3/2005 15:40'!
blockquote: aBlock
	self blockquote with: aBlock! !

!WAHtmlCanvas methodsFor: 'tags' stamp: 'lr 10/28/2007 14:42'!
break
	"Inserts a single line break."

	^ self brush: WABreakTag new! !

!WAHtmlCanvas methodsFor: 'tags-block' stamp: 'lr 10/28/2007 14:42'!
citation
	"Defines a citation."

	^ self tag: 'cite'! !

!WAHtmlCanvas methodsFor: 'tags-block' stamp: 'pmm 9/30/2006 20:06'!
citation: aBlock
	self citation with: aBlock! !

!WAHtmlCanvas methodsFor: 'tags-output' stamp: 'lr 10/28/2007 14:42'!
code
	"Defines computer code text."

	^ self tag: 'code'! !

!WAHtmlCanvas methodsFor: 'tags-output' stamp: 'avi 6/3/2005 15:40'!
code: aBlock
	self code with: aBlock! !

!WAHtmlCanvas methodsFor: 'tags-output' stamp: 'lr 10/28/2007 14:42'!
definition
	"Defines a definition term."

	^ self tag: 'dfn'! !

!WAHtmlCanvas methodsFor: 'tags-output' stamp: 'pmm 9/30/2006 20:06'!
definition: aBlock
	self definition with: aBlock! !

!WAHtmlCanvas methodsFor: 'tags-lists' stamp: 'lr 10/28/2007 14:42'!
definitionData
	"Defines a definition description."

	^ self tag: 'dd'! !

!WAHtmlCanvas methodsFor: 'tags-lists' stamp: 'pmm 5/31/2006 20:26'!
definitionData: aBlock
	self definitionData with: aBlock! !

!WAHtmlCanvas methodsFor: 'tags-lists' stamp: 'lr 10/28/2007 14:42'!
definitionList
	"Defines a definition list."

	^ self tag: 'dl'! !

!WAHtmlCanvas methodsFor: 'tags-lists' stamp: 'pmm 12/24/2005 17:42'!
definitionList: aBlock
	self definitionList with: aBlock! !

!WAHtmlCanvas methodsFor: 'tags-lists' stamp: 'lr 10/28/2007 14:42'!
definitionTerm
	"Defines a definition term."

	^ self tag: 'dt'! !

!WAHtmlCanvas methodsFor: 'tags-lists' stamp: 'pmm 12/24/2005 17:42'!
definitionTerm: aBlock
	self definitionTerm with: aBlock! !

!WAHtmlCanvas methodsFor: 'tags-block' stamp: 'lr 10/28/2007 14:42'!
deleted
	"Defines deleted text."

	^ self brush: (WAEditTag tag: 'del')! !

!WAHtmlCanvas methodsFor: 'tags-block' stamp: 'pmm 3/28/2007 10:42'!
deleted: aBlock
	self deleted with: aBlock! !

!WAHtmlCanvas methodsFor: 'tags-styles' stamp: 'lr 10/28/2007 14:42'!
div
	"Defines a section in a document."

	^ self brush: WADivTag new! !

!WAHtmlCanvas methodsFor: 'tags-styles' stamp: 'avi 6/3/2005 15:40'!
div: aBlock
	self div with: aBlock! !

!WAHtmlCanvas methodsFor: 'tags-format' stamp: 'lr 4/9/2007 16:53'!
emphasis
	"Defines emphasized text."

	^ self tag: 'em'! !

!WAHtmlCanvas methodsFor: 'tags-format' stamp: 'mb 3/8/2006 17:14'!
emphasis: aBlock
	self emphasis with: aBlock! !

!WAHtmlCanvas methodsFor: 'convenience' stamp: 'pmm 8/31/2008 14:34'!
encodeCharacter: aCharacter 
	self html: '&#', aCharacter asInteger seasideString, ';'! !

!WAHtmlCanvas methodsFor: 'tags-input' stamp: 'lr 10/28/2007 14:42'!
fieldSet
	"Defines a fieldset."

	^ self brush: WAFieldSetTag new! !

!WAHtmlCanvas methodsFor: 'tags-input' stamp: 'mb 3/24/2006 13:45'!
fieldSet: aBlock
	self fieldSet with: aBlock! !

!WAHtmlCanvas methodsFor: 'tags-input' stamp: 'lr 10/28/2007 14:42'!
form
	"Defines a form."

	^ self tag: 'form'! !

!WAHtmlCanvas methodsFor: 'tags-input' stamp: 'avi 6/3/2005 15:40'!
form: aBlock
	self form with: aBlock! !

!WAHtmlCanvas methodsFor: 'tags' stamp: 'lr 10/28/2007 14:42'!
heading
	"Defines header 1 to header 6."

	^ self brush: WAHeadingTag new! !

!WAHtmlCanvas methodsFor: 'tags' stamp: 'ac 3/16/2005 14:14'!
heading: aBlock
	self heading with: aBlock! !

!WAHtmlCanvas methodsFor: 'deprecated' stamp: 'jf 9/24/2008 10:51'!
heading: anObject level: anInteger
	self seasideDeprecatedApi: 'WAHtmlCanvas>>heading:level:'
		details: 'Use WAHeadingTag>>level: and WAHeadingTag>>with:. For example: "html heading level: 1; with: ''some text''".'.
	self heading level: anInteger; with: anObject! !

!WAHtmlCanvas methodsFor: 'tags' stamp: 'lr 10/28/2007 14:42'!
horizontalRule
	"Defines a horizontal rule."

	^ self brush: WAHorizontalRuleTag new! !

!WAHtmlCanvas methodsFor: 'public' stamp: 'jf 10/19/2008 03:37'!
html: aString 
	"Emit aString unescaped onto the target document."
	self flush.
	self document nextPutAll: aString seasideString! !

!WAHtmlCanvas methodsFor: 'tags-transitional' stamp: 'pmm 1/3/2008 16:59'!
iframe
	^ self brush: WAIframeTag new! !

!WAHtmlCanvas methodsFor: 'tags-images' stamp: 'lr 10/28/2007 14:42'!
image
	"Defines an image."

	^ self brush: WAImageTag new! !

!WAHtmlCanvas methodsFor: 'tags-images' stamp: 'mb 3/24/2006 13:51'!
image: aBlock
	self image with: aBlock! !

!WAHtmlCanvas methodsFor: 'deprecated' stamp: 'jf 9/24/2008 10:49'!
imageForm: aForm
	self seasideDeprecatedApi: 'WAHtmlCanvas>>imageForm:'
		details: 'Use WAImageTag>>form:. For example: "html image form: myForm".'.
	self image form: aForm! !

!WAHtmlCanvas methodsFor: 'tags-block' stamp: 'lr 4/9/2007 17:02'!
inserted
	"Defines inserted text."

	^ self brush: (WAEditTag tag: 'ins')! !

!WAHtmlCanvas methodsFor: 'tags-block' stamp: 'pmm 3/28/2007 10:41'!
inserted: aBlock
	self inserted with: aBlock! !

!WAHtmlCanvas methodsFor: 'tags-output' stamp: 'lr 10/28/2007 14:42'!
keyboard
	"Defines keyboard text."

	^ self tag: 'kbd'! !

!WAHtmlCanvas methodsFor: 'tags-output' stamp: 'lr 4/9/2007 17:23'!
keyboard: aBlock
	self keyboard with: aBlock! !

!WAHtmlCanvas methodsFor: 'deprecated' stamp: 'jf 9/24/2008 10:48'!
keyboardInput
	"Keyboard, text to be entered by the user.
	This selector has been renamed"

	self seasideDeprecatedApi: 'WAHtmlCanvas>>keyboardInput'
		details: 'WAHtmlCanvas>>keyboard'.
	^ self keyboard! !

!WAHtmlCanvas methodsFor: 'deprecated' stamp: 'jf 9/24/2008 10:47'!
keyboardInput: aBlock
	self seasideDeprecatedApi: 'WAHtmlCanvas>>keyboardInput:'
		details: 'WAHtmlCanvas>>keyboard:'.
	self keyboard: aBlock! !

!WAHtmlCanvas methodsFor: 'tags-input' stamp: 'lr 10/28/2007 14:42'!
label
	"Defines a label for a form control."

	^ self brush: WALabelTag new! !

!WAHtmlCanvas methodsFor: 'tags-input' stamp: 'avi 6/3/2005 15:40'!
label: aBlock
	self label with: aBlock! !

!WAHtmlCanvas methodsFor: 'tags-input' stamp: 'lr 10/28/2007 14:42'!
legend
	"Defines a title in a fieldset."

	^ self tag: 'legend'! !

!WAHtmlCanvas methodsFor: 'tags-input' stamp: 'mb 3/8/2006 18:00'!
legend: aBlock
	self legend with: aBlock! !

!WAHtmlCanvas methodsFor: 'tags-lists' stamp: 'lr 10/28/2007 14:42'!
listItem
	"Defines a list item."

	^ self tag: 'li'! !

!WAHtmlCanvas methodsFor: 'tags-lists' stamp: 'avi 6/3/2005 15:40'!
listItem: aBlock
	self listItem with: aBlock! !

!WAHtmlCanvas methodsFor: 'tags-images' stamp: 'lr 10/28/2007 14:42'!
map
	"Defines an anchor to be used around an image."

	^ self brush: WAImageMapTag new! !

!WAHtmlCanvas methodsFor: 'tags-images' stamp: 'lr 4/9/2007 17:47'!
map: aBlock
	self map with: aBlock! !

!WAHtmlCanvas methodsFor: 'tags-program' stamp: 'lr 10/28/2007 14:42'!
object
	"Defines an embedded object."

	^ self brush: WAObjectTag new! !

!WAHtmlCanvas methodsFor: 'tags-program' stamp: 'pmm 2/11/2007 19:31'!
object: aBlock
	self object with: aBlock! !

!WAHtmlCanvas methodsFor: 'tags-lists' stamp: 'lr 10/28/2007 14:42'!
orderedList
	"Defines an ordered list."

	^ self brush: WAOrderedListTag new! !

!WAHtmlCanvas methodsFor: 'tags-lists' stamp: 'lr 9/25/2005 11:52'!
orderedList: aBlock
	self orderedList with: aBlock! !

!WAHtmlCanvas methodsFor: 'tags' stamp: 'lr 10/28/2007 14:42'!
paragraph
	"Defines a paragraph."

	^ self tag: 'p'! !

!WAHtmlCanvas methodsFor: 'tags' stamp: 'avi 6/3/2005 15:40'!
paragraph: aBlock
	self paragraph with: aBlock! !

!WAHtmlCanvas methodsFor: 'tags-program' stamp: 'lr 10/28/2007 14:42'!
parameter
	"Defines a parameter for an object."

	^ self brush: WAParameterTag new! !

!WAHtmlCanvas methodsFor: 'tags-program' stamp: 'lr 2/13/2007 00:12'!
parameter: aBlock
	self parameter! !

!WAHtmlCanvas methodsFor: 'tags-input' stamp: 'ac 7/3/2005 15:46'!
popupAnchor
	^ self brush: WAPopupAnchorTag new! !

!WAHtmlCanvas methodsFor: 'tags-input' stamp: 'lr 7/10/2007 12:12'!
popupAnchor: aBlock
	self popupAnchor with: aBlock! !

!WAHtmlCanvas methodsFor: 'tags-output' stamp: 'lr 10/28/2007 14:42'!
preformatted
	"Defines preformatted text."

	^ self tag: 'pre'! !

!WAHtmlCanvas methodsFor: 'tags-output' stamp: 'avi 6/3/2005 15:40'!
preformatted: aBlock
	self preformatted with: aBlock! !

!WAHtmlCanvas methodsFor: 'tags-block' stamp: 'lr 10/28/2007 14:42'!
quote
	"Defines a short quotation."

	^ self tag: 'q'! !

!WAHtmlCanvas methodsFor: 'tags-block' stamp: 'pmm 9/30/2006 20:06'!
quote: aBlock
	self quote with: aBlock! !

!WAHtmlCanvas methodsFor: 'tags-ruby' stamp: 'mb 9/21/2006 16:26'!
ruby
	^ self tag: 'ruby'! !

!WAHtmlCanvas methodsFor: 'tags-ruby' stamp: 'mb 9/21/2006 16:26'!
ruby: aBlock
	self ruby with: aBlock! !

!WAHtmlCanvas methodsFor: 'tags-ruby' stamp: 'mb 9/21/2006 16:26'!
rubyBase
	^ self tag: 'rb'! !

!WAHtmlCanvas methodsFor: 'tags-ruby' stamp: 'mb 9/21/2006 16:26'!
rubyBase: aBlock
	self rubyBase with: aBlock! !

!WAHtmlCanvas methodsFor: 'tags-ruby' stamp: 'mb 9/21/2006 16:26'!
rubyBaseContainer
	^ self tag: 'rbc'! !

!WAHtmlCanvas methodsFor: 'tags-ruby' stamp: 'mb 9/21/2006 16:26'!
rubyBaseContainer: aBlock
	self rubyBaseContainer with: aBlock! !

!WAHtmlCanvas methodsFor: 'tags-ruby' stamp: 'mb 9/21/2006 16:26'!
rubyParentheses
	^ self tag: 'rp'! !

!WAHtmlCanvas methodsFor: 'tags-ruby' stamp: 'mb 9/21/2006 16:26'!
rubyParentheses: aBlock
	self rubyParentheses with: aBlock! !

!WAHtmlCanvas methodsFor: 'tags-ruby' stamp: 'mb 9/21/2006 16:26'!
rubyText
	^ self brush: WARubyTextTag new! !

!WAHtmlCanvas methodsFor: 'tags-ruby' stamp: 'mb 9/21/2006 16:26'!
rubyText: aBlock
	self rubyText with: aBlock! !

!WAHtmlCanvas methodsFor: 'tags-ruby' stamp: 'mb 9/21/2006 16:26'!
rubyTextContainer
	^ self tag: 'rtc'! !

!WAHtmlCanvas methodsFor: 'tags-ruby' stamp: 'mb 9/21/2006 16:26'!
rubyTextContainer: aBlock
	self rubyTextContainer with: aBlock! !

!WAHtmlCanvas methodsFor: 'tags-output' stamp: 'lr 10/28/2007 14:42'!
sample
	"Defines sample computer code."

	^ self tag: 'samp'! !

!WAHtmlCanvas methodsFor: 'tags-output' stamp: 'lr 4/9/2007 17:23'!
sample: aBlock
	self sample with: aBlock! !

!WAHtmlCanvas methodsFor: 'deprecated' stamp: 'jf 9/24/2008 10:45'!
sampleOutput
	"Sample output, from a program or script.
	This selector has been renamed."

	self seasideDeprecatedApi: 'WAHtmlCanvas>>sampleOutput'
		details: 'Use WAHtmlCanvas>>sample.'.
	^ self sample! !

!WAHtmlCanvas methodsFor: 'deprecated' stamp: 'jf 10/18/2008 21:05'!
sampleOutput: aBlock
	self seasideDeprecatedApi: 'WAHtmlCanvas>>sampleOutput:'
		details: 'Use WAHtmlCanvas>>sample:.'.
	self sample: aBlock! !

!WAHtmlCanvas methodsFor: 'tags-program' stamp: 'lr 10/28/2007 14:42'!
script
	"Defines a script."

	^ self brush: WAScriptTag new! !

!WAHtmlCanvas methodsFor: 'tags-program' stamp: 'avi 6/3/2005 15:40'!
script: aBlock
	self script with: aBlock! !

!WAHtmlCanvas methodsFor: 'tags-format' stamp: 'lr 10/28/2007 14:42'!
small
	"Defines small text."

	^ self tag: 'small'! !

!WAHtmlCanvas methodsFor: 'tags-format' stamp: 'avi 6/3/2005 15:40'!
small: aBlock
	self small with: aBlock! !

!WAHtmlCanvas methodsFor: 'private' stamp: 'lr 10/28/2007 14:42'!
spaceEntity
	"The HTML entity representing a space. To be subclassed as needed."

	^ '&nbsp;'! !

!WAHtmlCanvas methodsFor: 'tags-styles' stamp: 'lr 10/28/2007 14:42'!
span
	"Defines a section in a document."

	^ self tag: 'span'! !

!WAHtmlCanvas methodsFor: 'tags-styles' stamp: 'avi 6/3/2005 15:40'!
span: aBlock
	self span with: aBlock! !

!WAHtmlCanvas methodsFor: 'tags-format' stamp: 'lr 10/28/2007 14:42'!
strong
	"Defines strong text."

	^ self tag: 'strong'! !

!WAHtmlCanvas methodsFor: 'tags-format' stamp: 'ac 3/20/2005 01:24'!
strong: aBlock
	self strong with: aBlock! !

!WAHtmlCanvas methodsFor: 'tags-format' stamp: 'lr 10/28/2007 14:42'!
subscript
	"Defines subscripted text."

	^ self tag: 'sub'! !

!WAHtmlCanvas methodsFor: 'tags-format' stamp: 'lr 4/9/2007 16:46'!
subscript: aBlock
	self subscript with: aBlock! !

!WAHtmlCanvas methodsFor: 'tags-format' stamp: 'lr 10/28/2007 14:42'!
superscript
	"Defines superscripted text."

	^ self tag: 'sup'! !

!WAHtmlCanvas methodsFor: 'tags-format' stamp: 'lr 4/9/2007 16:46'!
superscript: aBlock
	self superscript with: aBlock! !

!WAHtmlCanvas methodsFor: 'tags-tables' stamp: 'lr 10/28/2007 14:42'!
table
	"Defines a table."

	^ self brush: WATableTag new! !

!WAHtmlCanvas methodsFor: 'tags-tables' stamp: 'lr 4/9/2007 17:05'!
table: aBlock
	self table with: aBlock! !

!WAHtmlCanvas methodsFor: 'tags-tables' stamp: 'lr 10/28/2007 14:42'!
tableBody
	"Defines a table body."

	^ self tag: 'tbody'! !

!WAHtmlCanvas methodsFor: 'tags-tables' stamp: 'avi 6/3/2005 15:40'!
tableBody: aBlock
	self tableBody with: aBlock! !

!WAHtmlCanvas methodsFor: 'tags-tables' stamp: 'lr 10/28/2007 14:42'!
tableCaption
	"Defines a table caption."

	^ self tag: 'caption'! !

!WAHtmlCanvas methodsFor: 'tags-tables' stamp: 'mb 12/8/2006 16:17'!
tableCaption: aBlock
	self tableCaption with: aBlock! !

!WAHtmlCanvas methodsFor: 'tags-tables' stamp: 'lr 10/28/2007 14:42'!
tableColumn
	"Defines attributes for table columns."

	^ self brush: WATableColumnTag new! !

!WAHtmlCanvas methodsFor: 'tags-tables' stamp: 'mb 12/7/2006 20:27'!
tableColumn: aBlock
	self tableColumn with: aBlock! !

!WAHtmlCanvas methodsFor: 'tags-tables' stamp: 'lr 10/28/2007 14:42'!
tableColumnGroup
	"Defines groups of table columns."

	^ self brush: WATableColumnGroupTag new! !

!WAHtmlCanvas methodsFor: 'tags-tables' stamp: 'mb 12/7/2006 20:27'!
tableColumnGroup: aBlock
	self tableColumnGroup with: aBlock! !

!WAHtmlCanvas methodsFor: 'tags-tables' stamp: 'lr 10/28/2007 14:42'!
tableData
	"Defines a table cell."

	^ self brush: WATableDataTag new! !

!WAHtmlCanvas methodsFor: 'tags-tables' stamp: 'avi 6/3/2005 15:40'!
tableData: aBlock
	self tableData with: aBlock! !

!WAHtmlCanvas methodsFor: 'tags-tables' stamp: 'lr 10/28/2007 14:42'!
tableFoot
	"Defines a table footer."

	^ self tag: 'tfoot'! !

!WAHtmlCanvas methodsFor: 'tags-tables' stamp: 'lr 4/9/2007 17:06'!
tableFoot: aBlock
	^ self tableFoot with: aBlock! !

!WAHtmlCanvas methodsFor: 'tags-tables' stamp: 'lr 10/28/2007 14:42'!
tableHead
	"Defines a table header."

	^ self tag: 'thead'! !

!WAHtmlCanvas methodsFor: 'tags-tables' stamp: 'avi 6/3/2005 15:40'!
tableHead: aBlock
	self tableHead with: aBlock! !

!WAHtmlCanvas methodsFor: 'tags-tables' stamp: 'lr 10/28/2007 14:42'!
tableHeading
	"Defines a table header."

	^ self brush: WATableHeadingTag new! !

!WAHtmlCanvas methodsFor: 'tags-tables' stamp: 'ac 3/16/2005 16:15'!
tableHeading: aBlock
	self tableHeading with: aBlock! !

!WAHtmlCanvas methodsFor: 'tags-tables' stamp: 'lr 10/28/2007 14:42'!
tableRow
	"Defines a table row."

	^ self tag: 'tr'! !

!WAHtmlCanvas methodsFor: 'tags-tables' stamp: 'avi 6/3/2005 15:40'!
tableRow: aBlock
	self tableRow with: aBlock! !

!WAHtmlCanvas methodsFor: 'tags-output' stamp: 'lr 10/28/2007 14:42'!
teletype
	"Defines teletype text."

	^ self tag: 'tt'! !

!WAHtmlCanvas methodsFor: 'tags-output' stamp: 'lr 4/9/2007 17:23'!
teletype: aBlock
	self teletype with: aBlock! !

!WAHtmlCanvas methodsFor: 'tags-input' stamp: 'pmm 11/4/2008 20:28'!
textArea
	^ self brush: WATextAreaTag new! !

!WAHtmlCanvas methodsFor: 'tags-lists' stamp: 'lr 10/28/2007 14:42'!
unorderedList
	"Defines an unordered list."

	^ self brush: WAUnorderedListTag new! !

!WAHtmlCanvas methodsFor: 'tags-lists' stamp: 'avi 6/3/2005 15:40'!
unorderedList: aBlock
	self unorderedList with: aBlock! !

!WAHtmlCanvas methodsFor: 'tags-output' stamp: 'lr 10/28/2007 14:42'!
variable
	"Defines a variable."

	^ self tag: 'var'! !

!WAHtmlCanvas methodsFor: 'tags-output' stamp: 'pmm 9/30/2006 20:06'!
variable: aBlock
	self variable with: aBlock! !

!WAHtmlCanvas methodsFor: 'convenience' stamp: 'avi 6/23/2006 10:16'!
withLineBreaksAndUrls: aString
	| stream |
	aString ifNil: [^ self].
	stream := aString readStream.
	[stream atEnd] whileFalse:
		[self withUrls: stream nextLine.
		stream atEnd ifFalse: [self break]]! !

!WAHtmlCanvas methodsFor: 'convenience' stamp: 'lr 10/28/2007 14:42'!
withUrls: aString
	| stream url |
	stream := aString readStream.
	[ stream atEnd ] whileFalse: [
		self text: (stream upToAndSkipThroughAll: 'http://').
			stream atEnd ifFalse: [
				url := 'http://', (stream upTo: Character space).
				self anchor
					url: url;
					with: url.
				self text: ' '] ]! !

WAHtmlCanvas subclass: #WARenderCanvas
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Seaside-Canvas'!

!WARenderCanvas commentStamp: 'pmm 4/20/2007 11:35' prior: 0!
This canvas knows about callbacks and is intertwined with the rest of the framework.!

!WARenderCanvas methodsFor: 'form' stamp: 'MAD 8/30/2008 12:35'!
basicForm
	^ self brush: WABasicFormTag new! !

!WARenderCanvas methodsFor: 'form-buttons' stamp: 'lr 10/28/2007 14:42'!
button
	^ self brush: WAButtonTag new! !

!WARenderCanvas methodsFor: 'form-buttons' stamp: 'lr 4/11/2008 09:03'!
button: anObject
	^ self button with: anObject! !

!WARenderCanvas methodsFor: 'form-buttons' stamp: 'lr 10/28/2007 14:42'!
cancelButton
	^ self brush: WACancelButtonTag new! !

!WARenderCanvas methodsFor: 'form-buttons' stamp: 'lr 4/11/2008 09:04'!
cancelButton: anObject
	^ self cancelButton with: anObject! !

!WARenderCanvas methodsFor: 'form-elements' stamp: 'lr 10/28/2007 14:42'!
checkbox
	^ self brush: WACheckboxTag new! !

!WARenderCanvas methodsFor: 'form-elements' stamp: 'mb 2/6/2006 10:09'!
dateInput
	^ self brush: WADateInput new! !

!WARenderCanvas methodsFor: 'form-elements' stamp: 'lr 10/28/2007 14:42'!
fileUpload
	^ self brush: WAFileUploadTag new! !

!WARenderCanvas methodsFor: 'form' stamp: 'MAD 8/30/2008 12:30'!
form
	^ self brush: WAFormTag new! !

!WARenderCanvas methodsFor: 'form-elements' stamp: 'lr 10/28/2007 14:42'!
hiddenInput
	^ self brush: WAHiddenInputTag new! !

!WARenderCanvas methodsFor: 'form-buttons' stamp: 'lr 10/28/2007 14:42'!
imageButton
	^ self brush: WAImageButtonTag new! !

!WARenderCanvas methodsFor: 'form-buttons' stamp: 'lr 4/11/2008 09:04'!
imageButton: anObject
	^ self imageButton with: anObject! !

!WARenderCanvas methodsFor: 'form-elements' stamp: 'pmm 11/11/2008 23:01'!
keyGenerator
	^ self brush: WAKeyGeneratorTag new! !

!WARenderCanvas methodsFor: 'form-elements' stamp: 'lr 10/28/2007 14:42'!
multiSelect
	^ self brush: WAMultiSelectTag new! !

!WARenderCanvas methodsFor: 'form-elements' stamp: 'lr 6/6/2007 18:00'!
multiSelect: aBlock
	^ self multiSelect with: aBlock! !

!WARenderCanvas methodsFor: 'form-elements' stamp: 'lr 10/28/2007 14:42'!
option
	^ self brush: WAOptionTag new! !

!WARenderCanvas methodsFor: 'form-elements' stamp: 'ac 3/17/2005 17:14'!
option: aBlock
	^ self option with: aBlock! !

!WARenderCanvas methodsFor: 'form-elements' stamp: 'lr 10/28/2007 14:42'!
optionGroup
	^ self brush: WAOptionGroupTag new! !

!WARenderCanvas methodsFor: 'form-elements' stamp: 'pmm 4/24/2007 10:52'!
optionGroup: aBlock
	^ self optionGroup with: aBlock! !

!WARenderCanvas methodsFor: 'deprecated' stamp: 'jf 9/28/2008 23:56'!
painter
	self seasideDeprecatedApi: 'WARenderCanvas>>painter'! !

!WARenderCanvas methodsFor: 'form-elements' stamp: 'lr 10/28/2007 14:42'!
passwordInput
	^ self brush: WAPasswordInputTag new! !

!WARenderCanvas methodsFor: 'form-elements' stamp: 'lr 4/11/2008 09:16'!
passwordInput: anObject
	^ self passwordInput with: anObject! !

!WARenderCanvas methodsFor: 'form-elements' stamp: 'lr 10/28/2007 14:42'!
radioButton
	^ self brush: WARadioButtonTag new! !

!WARenderCanvas methodsFor: 'form-elements' stamp: 'ac 7/6/2005 10:46'!
radioButton: aBlock
	^ self radioButton with: aBlock! !

!WARenderCanvas methodsFor: 'form-elements' stamp: 'ac 7/6/2005 11:29'!
radioGroup
	^ WARadioGroup canvas: self! !

!WARenderCanvas methodsFor: 'form-elements' stamp: 'ac 7/6/2005 10:44'!
radioGroup: aBlock
	^ aBlock value: self radioGroup! !

!WARenderCanvas methodsFor: 'form-elements' stamp: 'lr 10/28/2007 14:42'!
select
	^ self brush: WASelectTag new! !

!WARenderCanvas methodsFor: 'form-elements' stamp: 'pmm 4/24/2007 11:07'!
select: aBlock
	^ self select with: aBlock! !

!WARenderCanvas methodsFor: 'form-buttons' stamp: 'lr 10/28/2007 14:42'!
submitButton
	^ self brush: WASubmitButtonTag new! !

!WARenderCanvas methodsFor: 'form-buttons' stamp: 'lr 4/11/2008 09:05'!
submitButton: anObject
	^ self submitButton with: anObject! !

!WARenderCanvas methodsFor: 'form-elements' stamp: 'lr 4/11/2008 09:05'!
textArea: anObject
	^ self textArea with: anObject! !

!WARenderCanvas methodsFor: 'form-elements' stamp: 'lr 4/9/2007 17:20'!
textInput
	^ self brush: WATextInputTag new! !

!WARenderCanvas methodsFor: 'form-elements' stamp: 'lr 4/11/2008 09:06'!
textInput: anObject
	^ self textInput with: anObject! !

!WARenderCanvas methodsFor: 'form-elements' stamp: 'mb 2/6/2006 10:10'!
timeInput
	^ self brush: WATimeInput new! !

WAHtmlCanvas subclass: #WAStaticHtmlCanvas
	instanceVariableNames: 'document'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Seaside-Canvas'!

!WAStaticHtmlCanvas commentStamp: 'pmm 4/15/2007 22:10' prior: 0!
This canvas can be used to generate static/RESTful HTML pages.!

!WAStaticHtmlCanvas methodsFor: 'accessing' stamp: 'avi 9/10/2005 17:20'!
document
	^ document! !

!WAStaticHtmlCanvas methodsFor: 'accessing' stamp: 'ac 6/9/2006 12:42'!
document: aDocument
	document := aDocument! !

!WAStaticHtmlCanvas methodsFor: 'initialize-release' stamp: 'avi 9/10/2005 17:20'!
initializeWithDocument: aDocument
	document := aDocument! !

!WATagCanvas methodsFor: 'private' stamp: 'jf 10/19/2008 03:27'!
break
	self subclassResponsibility ! !

!WATagCanvas methodsFor: 'convenience' stamp: 'jf 10/19/2008 03:37'!
space
	self flush.
	self document nextPutAll: self spaceEntity! !

!WATagCanvas methodsFor: 'convenience' stamp: 'jf 10/19/2008 03:18'!
space: anInteger
	anInteger timesRepeat: [ self space ]! !

!WATagCanvas methodsFor: 'private' stamp: 'jf 10/19/2008 03:15'!
spaceEntity
	self subclassResponsibility ! !

!WATagCanvas methodsFor: 'public' stamp: 'jf 10/19/2008 03:13'!
tag: aString
	"Defines a generic tag with the name aString."

	^ self brush: (WAGenericTag tag: aString)! !

!WATagCanvas methodsFor: 'public' stamp: 'pmm 11/5/2008 14:10'!
tag: aString with: aBlock
	(self tag: aString) with: aBlock! !

!WATagCanvas methodsFor: 'convenience' stamp: 'jf 10/19/2008 03:26'!
withLineBreaks: aString
	"Renders text preserving line breaks."
	| stream |
	stream := aString readStream.
	[ stream atEnd ] whileFalse: [
		self text: stream nextLine.
		stream atEnd ifFalse: [ self break ] ]! !

WATagCanvas subclass: #WAXmlCanvas
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Seaside-Canvas'!

!WAXmlCanvas commentStamp: 'pmm 9/12/2008 18:17' prior: 0!
This is an abstract canvas base class for build canvae for custom XML dialects.!

!WAXmlCanvas methodsFor: 'private' stamp: 'jf 10/19/2008 11:23'!
break
	^ String with: Character cr with: Character lf! !

!WAXmlCanvas methodsFor: 'convenience' stamp: 'jf 10/19/2008 03:20'!
cdata: aString
	"aString must not include ']]>'
	
	Google says if there is an error in the feed, there is a 0.02% chance
	that this is the cause, combined with the fact that about 7% of all feeds
	have errors there is a (2 / 10000) * (7 / 100) chance this goes wrong.
	Do you deal in chance?
	
	http://googlereader.blogspot.com/2005/12/xml-errors-in-feeds.html"
	self flush.
	self document nextPutAll: '<!![CDATA['.
	self document nextPutAll: aString.
	self document nextPutAll: ']]>'! !

!WAXmlCanvas methodsFor: 'private' stamp: 'jf 10/19/2008 03:17'!
spaceEntity
	^'&#160;'! !

!WARequestContext methodsFor: '*Seaside-Canvas' stamp: 'jf 10/18/2008 16:45'!
closeThisPopup
	^ self respond:
		(WAResponse new
			nextPutAll: '<html><script>';
			nextPutAll: 'self.close();';
			nextPutAll: '</script></html>';
			yourself)! !

!WARequestContext methodsFor: '*Seaside-Canvas' stamp: 'jf 10/18/2008 16:45'!
closeThisPopupAndReloadOpener
	^ self respond:
		(WAResponse new
			nextPutAll: '<html><script>';
			nextPutAll: 'self.close();';
			nextPutAll: 'self.opener.location=self.opener.location';
			nextPutAll: '</script></html>';
			yourself)! !

!WAPackage class methodsFor: '*Seaside-Canvas' stamp: 'lr 10/23/2008 22:02'!
seasideCanvas
	^ self new
		name: 'Seaside-Canvas';
		description: 'The mechanism of choice to generate valid and reusable XHTML markup.';
		addDependency: 'Seaside-Core';
		yourself
! !
